@@ -18,7 +18,7 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  *
- * $Id: nas.c,v 1.8 2006/07/11 16:12:26 hanke Exp $
+ * $Id: nas.c,v 1.8 2006-07-11 16:12:26 hanke Exp $
  */
 
 /* Internal event handler */
@@ -197,4 +197,4 @@ nas_set_volume(AudioID*id, int volume)
 }
 
 /* Provide the NAS backend */
-AudioFunctions nas_functions = {nas_open, nas_play, nas_stop, nas_close, nas_set_volume};
+spd_audio_plugin_t nas_functions = {nas_open, nas_play, nas_stop, nas_close, nas_set_volume};
