@@ -19,9 +19,12 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  *
- * $Id: spd_audio.h,v 1.18 2007-12-08 21:58:40 gcasse Exp $
+ * $Id: spd_audio.h,v 1.21 2008-10-15 17:28:17 hanke Exp $
  */
 
+#ifndef __SPD_AUDIO_H
+#define __SPD_AUDIO_H
+
 #include <pthread.h>
 #include <sys/types.h>
 
@@ -35,30 +38,15 @@
 #endif
 
 #ifdef WITH_PULSE
-#include <pulse/pulseaudio.h>
-#include <stdio.h> // TBD
-#include <semaphore.h>
-
-typedef struct {
-  unsigned int my_time_start;
-  unsigned int my_time_start_old;
-  int my_close_is_imminent;
-  sem_t my_sem;
-  pthread_t my_thread;
-} t_pulse_timeout;
-
+#include <pulse/simple.h>
 #endif
 
 #define AUDIO_BUF_SIZE 4096
 
-typedef enum{AUDIO_OSS = 0, AUDIO_NAS = 1, AUDIO_ALSA=2, AUDIO_PULSE=3} AudioOutputType;
+typedef enum{AUDIO_OSS = 0, AUDIO_NAS = 1, AUDIO_ALSA=2, AUDIO_PULSE=3, AUDIO_LIBAO=4} AudioOutputType;
 typedef enum{SPD_AUDIO_LE, SPD_AUDIO_BE} AudioFormat;
 
-#if defined(BYTE_ORDER) && BYTE_ORDER == BIG_ENDIAN
-AudioFormat audio_endian = SPD_AUDIO_BE;
-#else
-AudioFormat audio_endian = SPD_AUDIO_LE;
-#endif
+AudioFormat spd_audio_endian;
 
 typedef struct{
     int bits;
@@ -69,13 +57,7 @@ typedef struct{
     signed short *samples;
 }AudioTrack;
 
-typedef struct{
-    int   (* open)  (void *id, void** pars);
-    int   (* play)  (void *id, AudioTrack track);
-    int   (* stop)  (void *id);
-    int   (* close) (void *id);
-    int   (* set_volume) (void *id, int);
-}Funct;
+struct spd_audio_plugin;
 
 typedef struct{
     AudioOutputType type;
@@ -97,6 +79,7 @@ typedef struct{
     snd_pcm_sw_params_t *alsa_sw_params;	/* parameters of playback */
     snd_pcm_uframes_t alsa_buffer_size;
     pthread_mutex_t alsa_pcm_mutex;	/* mutex to guard the state of the device */
+    pthread_mutex_t alsa_pipe_mutex;	/* mutex to guard the stop pipes */
     int alsa_stop_pipe[2];		/* Pipe for communication about stop requests*/
     int alsa_fd_count;		/* Counter of descriptors to poll */
     struct pollfd *alsa_poll_fds; /* Descriptors to poll */
@@ -110,50 +93,33 @@ typedef struct{
     pthread_mutex_t flow_mutex;
     pthread_t nas_event_handler;
 #endif
-
 #ifdef WITH_PULSE
-    pa_context *pulse_context;
-    pa_stream *pulse_stream;
-    pa_threaded_mainloop *pulse_mainloop;
-    pa_cvolume pulse_volume;
-    int pulse_volume_valid;
-    int pulse_do_trigger;
-    int pulse_time_offset_msec;
-    int pulse_just_flushed;
-    int pulse_connected;
-    int pulse_success; // status for synchronous operation */
-    int pulse_stop_required;
-    pthread_mutex_t pulse_mutex;
-    pa_time_event *pulse_volume_time_event;
-    int pulse_max_length;
-    int pulse_target_length;
-    int pulse_pre_buffering;
-    int pulse_min_request;
-    char* pulse_server;
-    t_pulse_timeout pulse_timeout;  
+    pa_simple *pa_simple;
+    char *pa_server;
+    int pa_min_audio_length;
+    volatile int pa_stop_playback;
+    int pa_current_rate;  // Sample rate for currently PA connection
+    int pa_current_bps; // Bits per sample rate for currently PA connection
+    int pa_current_channels; // Number of channels for currently PA connection
 #endif
 
-    Funct *function;
+    struct spd_audio_plugin *function;
 
     int working;
 }AudioID;
 
-typedef int (* t_audio_open)(AudioID *id, void** pars);
-typedef int (* t_audio_play)(AudioID *id, AudioTrack track);
-typedef int (* t_audio_stop)(AudioID *id);
-typedef int (* t_audio_close)(AudioID *id);
-typedef int (* t_audio_set_volume)(AudioID *id, int volume);
-
-typedef struct{
-    t_audio_open open;
-    t_audio_play play;
-    t_audio_stop stop;
-    t_audio_close close;
-    t_audio_set_volume set_volume;
-}AudioFunctions;    
+typedef struct spd_audio_plugin {
+    int   (* open)  (AudioID *id, void** pars);
+    int   (* play)  (AudioID *id, AudioTrack track);
+    int   (* stop)  (AudioID *id);
+    int   (* close) (AudioID *id);
+    int   (* set_volume) (AudioID *id, int);
+} spd_audio_plugin_t;
 
 AudioID* spd_audio_open(AudioOutputType type, void **pars, char **error);
 
+int log_level;
+
 int spd_audio_play(AudioID *id, AudioTrack track, AudioFormat format);
 
 int spd_audio_stop(AudioID *id);
@@ -161,3 +127,7 @@ int spd_audio_stop(AudioID *id);
 int spd_audio_close(AudioID *id);
 
 int spd_audio_set_volume(AudioID *id, int volume);
+
+void spd_audio_set_loglevel(int level);
+
+#endif /* ifndef #__SPD_AUDIO_H */
