@@ -40,6 +40,7 @@
 #include <glib.h>
 #include <errno.h>
 #include <assert.h>
+#include <netdb.h>
 
 #include "def.h"
 #include "libspeechd.h"
@@ -50,16 +51,12 @@
 /* --------------  Private functions headers ------------------------*/
 
 #ifdef LIBSPEECHD_DEBUG
-FILE *spd_debug;
+FILE *spd_debug = NULL;
 #endif
 
-typedef struct {
-    char *str;
-    size_t bytes;
-}_SPDString;
 
 static int spd_set_priority(SPDConnection* connection, SPDPriority priority);
-static char* escape_dot(const char *otext);
+static char* escape_dot(const char *text);
 static int isanum(char* str);		
 static char* get_reply(SPDConnection *connection);
 static int get_err_code(char *reply);
@@ -70,12 +67,10 @@ static void xfree(void *ptr);
 static int ret_ok(char *reply);
 static void SPD_DBG(char *format, ...);
 static void* spd_events_handler(void*);
-static _SPDString spd_string_append(_SPDString str, char *tail, int bytes);
-static _SPDString spd_string_new(void);
 
 pthread_mutex_t spd_logging_mutex;
 
-#if !(defined(__GLIBC__) || defined(_GNU_SOURCE))
+#if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
 /* Added by Willie Walker - strndup and getline are gcc-isms */
 char *strndup ( const char *s, size_t n)
 {
@@ -146,6 +141,83 @@ ssize_t getline (char **lineptr, size_t *n, FILE *f)
 
 #define SPD_REPLY_BUF_SIZE 65536
 
+/* Determine address for the unix socket */
+static char*
+_get_default_unix_socket_name(void)
+{
+  GString* socket_filename;
+  char *h;
+  const char *homedir = g_getenv("HOME");
+  if (!homedir)
+    homedir = g_get_home_dir();
+  socket_filename = g_string_new("");
+  g_string_printf(socket_filename, "%s/.speech-dispatcher/speechd.sock", homedir);
+  // Do not regurn glib string, but glibc string...
+  h = strdup(socket_filename->str);
+  g_string_free(socket_filename, 1);
+  return h;
+}
+
+SPDConnectionAddress*
+spd_get_default_address(char **error)
+{
+  const gchar *env_address = g_getenv("SPEECHD_ADDRESS");
+  gchar **pa; /* parsed address */
+  SPDConnectionAddress *address = malloc(sizeof(SPDConnectionAddress));
+
+  if (env_address == NULL){ // Default method = unix sockets
+    address->method = SPD_METHOD_UNIX_SOCKET;
+    address->unix_socket_name = _get_default_unix_socket_name();
+  }else{
+    pa = g_strsplit(env_address, ":", 0);
+    assert (pa);
+    if (!g_strcmp0(pa[0], "unix_socket") || pa[0] == NULL){ // Unix sockets
+      address->method = SPD_METHOD_UNIX_SOCKET;
+      if (pa[1] == NULL){
+	address->unix_socket_name = _get_default_unix_socket_name();
+      }else{
+	address->unix_socket_name = strdup(pa[1]);
+      }
+    }else if (!g_strcmp0(pa[0], "inet_socket")){ // Inet sockets
+      address->method = SPD_METHOD_INET_SOCKET;
+      if (pa[1] == NULL){
+	address->inet_socket_host = strdup("127.0.0.1");
+	address->inet_socket_port = 6560;
+      }else{
+	address->inet_socket_host = strdup(pa[1]);
+	if (pa[2] == NULL){
+	  address->inet_socket_port = SPEECHD_DEFAULT_PORT;
+	}else{
+	  address->inet_socket_port = atoi(pa[2]);
+        }
+      }
+    }else{ // Unknown or unsupported method requested
+      *error = strdup("Unknown or unsupported communication method");
+      address = NULL;
+    }
+    g_strfreev(pa);
+  }
+  return address;
+}
+
+
+static void _init_debug(void)
+{
+#ifdef LIBSPEECHD_DEBUG
+  if (!spd_debug){
+    spd_debug = fopen("/tmp/libspeechd.log", "w");
+    if (spd_debug == NULL) SPD_FATAL("COULDN'T ACCES FILE INTENDED FOR DEBUG");
+     
+    if(pthread_mutex_init(&spd_logging_mutex, NULL)){
+      fprintf(stderr, "Mutex initialization failed");
+      fflush(stderr);
+      exit(1);
+    }
+    SPD_DBG("Debugging started");
+  }
+#endif /* LIBSPEECHD_DEBUG */
+}
+
 /* Opens a new Speech Dispatcher connection.
  * Returns socket file descriptor of the created connection
  * or -1 if no connection was opened. */
@@ -154,32 +226,152 @@ SPDConnection*
 spd_open(const char* client_name, const char* connection_name, const char* user_name,
 	 SPDConnectionMode mode)
 {
-    return spd_open2(client_name, connection_name, user_name,
-		     mode, SPD_METHOD_UNIX_SOCKET, 1);
+  char *error;
+  int autospawn = 1;
+  SPDConnection *conn;
+  conn = spd_open2(client_name, connection_name, user_name,
+		   mode, NULL, autospawn, &error);
+  if (!conn){
+    _init_debug();
+    assert(error);
+    SPD_DBG("Could not connect to Speech Dispatcher: %s", error);
+    xfree(error);
+  }
+  return conn;
+}
+
+#define MAX_IP_SIZE 16+1
+/* TODO: This only works in IPV4 */
+static char*
+resolve_host(char* host_name_or_ip, int *is_localhost, gchar **error)
+{
+    struct addrinfo *addr_result;
+    int err;
+    char *resolve_buffer = malloc(MAX_IP_SIZE*sizeof(char));
+    const char *resolved_ip = NULL;
+    char *ip;
+    *error = NULL;
+    struct sockaddr_in *addr_in;
+    
+    if (resolve_buffer == NULL) {
+	*error = g_strdup("Failed to allocate memory.");
+	return NULL;
+    }
+
+    err = getaddrinfo(host_name_or_ip, 0, NULL, &addr_result);
+    if (err){
+	*error = g_strdup_printf("Can't resolve address %d due to error %s:",
+				 err, gai_strerror(err));
+	xfree(resolve_buffer);
+	return NULL;
+    }
+    /* Take the first address returned as we are only interested in host ip */
+    addr_in = (struct sockaddr_in *) addr_result->ai_addr;
+    resolved_ip = inet_ntop(AF_INET, &(addr_in->sin_addr.s_addr), resolve_buffer, MAX_IP_SIZE);
+    if (resolved_ip == NULL) {
+	*error = g_strdup_printf("Could not convert address, due to the following error: %s",
+				 strerror(errno));
+	freeaddrinfo(addr_result);
+	xfree(resolve_buffer);
+	return NULL;
+    }
+
+    if (!strncmp(resolved_ip, "127.",4)){
+	*is_localhost = 1;
+	/* In case of local addresses, use 127.0.0.1 which is guaranteed
+	   to be local and the server listens on it */
+	xfree(resolve_buffer);
+	ip = strdup("127.0.0.1");
+    }else{
+	*is_localhost = 0;
+	ip = resolve_buffer;
+    }
+    freeaddrinfo(addr_result);
+    return ip;
+}
+
+static int
+spawn_server(SPDConnectionAddress *address, int is_localhost, gchar **spawn_error)
+{
+    gchar **speechd_cmd = malloc(16*sizeof(char*));
+    gchar *stderr_output;
+    gboolean spawn_ok;
+    GError *gerror = NULL;
+    int exit_status;
+    int i;
+    char *resolved_ip;
+
+    if ((address->method==SPD_METHOD_INET_SOCKET) && (!is_localhost)){
+	*spawn_error = g_strdup("Spawn failed, the given network address doesn't seem to be on localhost");
+	return 1;
+    }
+
+    speechd_cmd[0] = g_strdup(SPD_SPAWN_CMD);
+    speechd_cmd[1] = g_strdup("--spawn");
+    speechd_cmd[2] = g_strdup("--communication-method");
+    if (address->method==SPD_METHOD_INET_SOCKET){
+	speechd_cmd[3] = g_strdup("inet_socket");
+	speechd_cmd[4] = g_strdup("--port");
+	speechd_cmd[5] = g_strdup_printf("%d",address->inet_socket_port);
+	speechd_cmd[6] = NULL;
+    }else if (address->method==SPD_METHOD_UNIX_SOCKET){
+	speechd_cmd[3] = g_strdup("unix_socket");
+	speechd_cmd[4] = g_strdup("--socket-path");
+	speechd_cmd[5] = g_strdup_printf("%s", address->unix_socket_name);
+	speechd_cmd[6] = NULL;
+    }else assert (0);
+    
+    spawn_ok = g_spawn_sync(NULL, (gchar**)speechd_cmd, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL,
+			    NULL, NULL, NULL, &stderr_output, &exit_status, &gerror);
+    for (i=0;speechd_cmd[i]!=NULL;i++)
+	g_free(speechd_cmd[i]);
+    if (!spawn_ok){
+	*spawn_error = g_strdup_printf("Autospawn failed. Spawn error %d: %s", gerror->code, gerror->message);
+	return 1;
+    }else{
+	if (exit_status){
+	    *spawn_error = g_strdup_printf("Autospawn failed. Speech Dispatcher refused to start with error code, " \
+					   "stating this as a reason: %s", stderr_output);
+	    return 1;
+	}else{
+	    *spawn_error = NULL;
+	    return 0;
+	}
+    }
+    assert(0);
 }
 
 SPDConnection*
 spd_open2(const char* client_name, const char* connection_name, const char* user_name,
-	  SPDConnectionMode mode, SPDConnectionMethod method, int autospawn)
+	  SPDConnectionMode mode, SPDConnectionAddress *address, int autospawn,
+	  char **error_result)
 {
     SPDConnection *connection;
     char *set_client_name;
     char* conn_name;
     char* usr_name;
-    char *env_port;
-    char *env_socket;
-    int port;
     int ret;
     char tcp_no_delay = 1;
 
     /* Autospawn related */
-    const char *pidof_speechd[] = { "pidof", "speech-dispatcher", NULL };
-    const char *speechd_cmd[] = { SPD_SPAWN_CMD, "--spawn", NULL };
-    gchar *speechd_pid = NULL;
-    GError *error = NULL;
-    gint exit_status;
+    int spawn_err;
+    gchar *spawn_report;
+    char *host_ip;
+    int is_localhost = 1;
+    
+    struct sockaddr_in address_inet;    
+    struct sockaddr_un address_unix;    
+    struct sockaddr *sock_address;
+    size_t sock_address_len;
+    gchar *resolve_error;
 
-    if (client_name == NULL) return NULL;
+    _init_debug();
+
+    if (client_name == NULL){
+      *error_result = strdup("ERROR: Client name not specified");
+      SPD_DBG(*error_result);
+      return NULL;
+    }
     
     if (user_name == NULL)
     {
@@ -192,81 +384,72 @@ spd_open2(const char* client_name, const char* connection_name, const char* user
         conn_name = strdup("main");
     else
         conn_name = strdup(connection_name);
-    
-#ifdef LIBSPEECHD_DEBUG
-    spd_debug = fopen("/tmp/libspeechd.log", "w");
-    if (spd_debug == NULL) SPD_FATAL("COULDN'T ACCES FILE INTENDED FOR DEBUG");
 
-    ret = pthread_mutex_init(&spd_logging_mutex, NULL);
-    if(ret != 0){
-      fprintf(stderr, "Mutex initialization failed");
-      fflush(stderr);
-      exit(1);
-    }
-    SPD_DBG("Debugging started");
-#endif /* LIBSPEECHD_DEBUG */
-    
-    /* Autospawn -- check if Dispatcher is not running and if so, start it*/
-    if (autospawn){
-      if (g_spawn_sync(NULL, (gchar**)pidof_speechd, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &speechd_pid, NULL,
-		       &exit_status, &error) && strlen(speechd_pid) == 0){
-	g_spawn_sync(NULL, (gchar**)speechd_cmd, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
-		     NULL, NULL, NULL, NULL, &exit_status, &error);
+    if (address == NULL){
+      char *err = NULL;
+      address = spd_get_default_address(&err);
+      if (!address){
+	assert(err);
+	*error_result = err;
+	SPD_DBG(*error_result);
+	return NULL;
       }
     }
-
+    
     /* Connect to server using the selected method */
     connection = xmalloc(sizeof(SPDConnection));
-    if (method==SPD_METHOD_INET_SOCKET){    
-      struct sockaddr_in address_inet;
-
-      env_port = getenv("SPEECHD_PORT");
-      if (env_port != NULL)
-        port = strtol(env_port, NULL, 10);
-      else
-        port = SPEECHD_DEFAULT_PORT;
-    
-      address_inet.sin_addr.s_addr = inet_addr("127.0.0.1");
-      address_inet.sin_port = htons(port);
-      address_inet.sin_family = AF_INET;
-      connection->socket = socket(AF_INET, SOCK_STREAM, 0);
-      ret = connect(connection->socket, (struct sockaddr *)&address_inet, sizeof(address_inet));
-      if (ret == -1){
-        SPD_DBG("Error: Can't connect to localhost on port %d using inet sockets: %s", port, strerror(errno));
-        close(connection->socket);
-	return NULL;
-      }
-      setsockopt(connection->socket, IPPROTO_TCP, TCP_NODELAY, &tcp_no_delay, sizeof(int));
-
-    }else if (method==SPD_METHOD_UNIX_SOCKET){
-      struct sockaddr_un address_unix;
-      GString* socket_filename;
-      /* Determine address for the unix socket */
-      env_socket = getenv("SPEECHD_SOCKET");
-      if (env_socket){
-	socket_filename = g_string_new(env_socket);
-      }else{
-	const char *homedir = g_getenv("HOME");
-	if (!homedir)
-	  homedir = g_get_home_dir();
-	socket_filename = g_string_new("");
-	g_string_printf(socket_filename, "%s/.speech-dispatcher/speechd.sock", homedir);
-      }
+    if (address->method==SPD_METHOD_INET_SOCKET){    
+	host_ip = resolve_host(address->inet_socket_host, &is_localhost, &resolve_error);
+	if (host_ip == NULL){
+	    *error_result = strdup(resolve_error);
+	    g_free(resolve_error);
+	    return NULL;
+	}	
+	address_inet.sin_addr.s_addr = inet_addr(host_ip);
+	address_inet.sin_port = htons(address->inet_socket_port);
+	address_inet.sin_family = AF_INET;
+	connection->socket = socket(AF_INET, SOCK_STREAM, 0);
+	sock_address = (struct sockaddr*) &address_inet;
+	sock_address_len = sizeof(address_inet);
+    }else if (address->method==SPD_METHOD_UNIX_SOCKET){
       /* Create the unix socket */
       address_unix.sun_family = AF_UNIX;
-      strncpy (address_unix.sun_path, socket_filename->str, sizeof (address_unix.sun_path));
+      strncpy (address_unix.sun_path, address->unix_socket_name, sizeof (address_unix.sun_path));
       address_unix.sun_path[sizeof (address_unix.sun_path) - 1] = '\0';
       connection->socket = socket(AF_UNIX, SOCK_STREAM, 0);
-      ret = connect(connection->socket, (struct sockaddr *)&address_unix, SUN_LEN(&address_unix));
-      if (ret == -1){
-        SPD_DBG("Error: Can't connect to unix socket %s: %s", socket_filename->str, strerror(errno));
-        close(connection->socket);
-	return NULL;
-      }
-      /* Clean up */
-      g_string_free(socket_filename, 0);
-    }else SPD_FATAL("Unsupported connection method to spd_open");
+      sock_address = (struct sockaddr*) &address_unix;
+      sock_address_len = SUN_LEN(&address_unix);
+    }else SPD_FATAL("Unsupported connection method for spd_open2()");
 
+    ret = connect(connection->socket, sock_address, sock_address_len);    
+    if (ret == -1){
+	/* Suppose server might not be running, try to autospawn (autostart) it */
+	if (autospawn){	
+	    spawn_err = spawn_server(address, is_localhost, &spawn_report);
+	    if (!spawn_err)
+		spawn_report = g_strdup("Server successfully autospawned");
+	    ret = connect(connection->socket, sock_address, sock_address_len);    
+	}else{
+	    spawn_report = g_strdup("Autospawn disabled");
+	}
+	if (ret == -1){
+	    if (address->method == SPD_METHOD_INET_SOCKET)
+		*error_result = g_strdup_printf("Error: Can't connect to %s on port %d using inet sockets: %s. " \
+						"Autospawn: %s", address->inet_socket_host,
+						address->inet_socket_port, strerror(errno), spawn_report);
+	    else if (address->method == SPD_METHOD_UNIX_SOCKET)
+		*error_result = g_strdup_printf("Error: Can't connect to unix socket %s: %s. Autospawn: %s",
+						address->unix_socket_name, strerror(errno), spawn_report);
+	    else assert (0);
+	    SPD_DBG(*error_result);
+	    close(connection->socket);	
+	    return NULL;
+	}
+    }
+
+    if (address->method == SPD_METHOD_INET_SOCKET)
+	setsockopt(connection->socket, IPPROTO_TCP, TCP_NODELAY, &tcp_no_delay, sizeof(int));
+    
     connection->callback_begin = NULL;
     connection->callback_end = NULL;
     connection->callback_im = NULL;
@@ -299,20 +482,17 @@ spd_open2(const char* client_name, const char* connection_name, const char* user
 	pthread_mutex_init(connection->mutex_reply_ack, NULL);
 	ret = pthread_create(connection->events_thread, NULL, spd_events_handler, connection);
 	if(ret != 0){
-	    SPD_DBG("Thread initialization failed");
+	    *error_result = strdup("Thread initialization failed");
+	    SPD_DBG(*error_result);
 	    return NULL;
 	}
     }
 
     /* By now, the connection is created and operational */
-
     set_client_name = g_strdup_printf("SET SELF CLIENT_NAME \"%s:%s:%s\"", usr_name,
 				      client_name, conn_name);
-
     ret = spd_execute_command_wo_mutex(connection, set_client_name);   
-
     xfree(usr_name);  xfree(conn_name);  xfree(set_client_name);
-
     return connection;
 }
 
@@ -1098,7 +1278,11 @@ spd_send_data_wo_mutex(SPDConnection *connection, const char *message, int wfr)
     }
     /* write message to the socket */
     SPD_DBG("Writing to socket");
-    write(connection->socket, message, strlen(message));
+    if(!write(connection->socket, message, strlen(message))){
+	SPD_DBG("Can't write to socket: %s", strerror(errno));
+	pthread_mutex_unlock(connection->mutex_reply_ready);
+	return NULL;
+    }
     SPD_DBG("Written to socket");
     SPD_DBG(">> : |%s|", message);
 
@@ -1464,90 +1648,58 @@ xfree(void *ptr)
         free(ptr);
 }
 
-static _SPDString
-spd_string_new(void)
-{
-    _SPDString result;
-    
-    result.bytes=1024;
-    result.str=malloc(result.bytes);
-    result.str[0]='\0';
-    
-    return result;
-}
-
-static _SPDString
-spd_string_append(_SPDString str, char *tail, int bytes)
-{
-    if (tail == NULL) return str;
-    
-    while ((strlen(str.str)+strlen(tail))>= str.bytes){
-	str.bytes *= 2;
-	str.str = realloc(str.str, str.bytes);
-    }
-
-    if (bytes != -1)
-	strncat(str.str, tail, bytes);
-    else
-	strcat(str.str, tail);
-
-    return str;
-}   
-
+/*
+ * escape_dot: Replace . with .. at the start of lines.
+ * @text: text to escape
+ * @Returns: An allocated string, containing the escaped text.
+ */
 static char*
 escape_dot(const char *text)
 {
-    _SPDString result;
-    size_t text_len;
-    int pos;
-    char *place;
+    size_t orig_len = 0;
+    const char *orig_end;
+    char *result = NULL;
+    char *result_ptr;
+    static const char *ESCAPED_DOTLINE = "\r\n..";
+    static const size_t ESCAPED_DOTLINELEN = 4;
+    static const size_t DOTLINELEN = 3;
 
-    result=spd_string_new();
+    if (text == NULL)
+        return NULL;
 
-    if (text == NULL) return NULL;
-    text_len = strlen(text);
+orig_len = strlen(text);
+    orig_end = text + orig_len;
+    result = malloc((orig_len * 2 + 1) * sizeof(char));
 
-    pos = 0;
-    if (text_len == 0){
-	return result.str;
-    }
-    /*If text contains only a dot, escape it*/
-    else if (text_len==1 && text[0]=='.'){
-	result = spd_string_append(result, "..", -1);
-	return result.str;
-    }
-    /*Escape the leading '.\r\n', if present*/
-    else if (text_len >=3){
-	if (!strncmp(text, ".\r\n", 3)){
-	    result = spd_string_append(result, "..", -1);
-	    pos += 1;
-	}
+    if (result == NULL)
+        return NULL;
+
+    result_ptr = result;
+
+    /* We're over-allocating.  Even if we replaced every character
+     * in text with "..", the length of the escaped string can be no more
+     * than orig_len * 2.  We could tighten that upper bound with
+     * a little more work.
+     */
+
+    if ((orig_len >= 1) && (text[0] == '.')) {
+        *(result_ptr++) = '.';
+        *(result_ptr++) = '.';
+        text += 1;
     }
 
-    /* Replace all full \r\n.\r\n sequences */
-    while((place = strstr(&text[pos], "\r\n.\r\n"))){
-	assert (place >= text);
-	int n = place-&text[pos];
-	result = spd_string_append(result, (char*) &text[pos], n);
-	result = spd_string_append(result, "\r\n..", -1);
-	/* Only rewind before the following \r\n to resume search for
-	   sequence */
-	pos +=n+3;
+    while (text < orig_end) {
+        if ((text[0] == '\r') && (text[1] == '\n') && (text[2] == '.')) {
+            memcpy(result_ptr, ESCAPED_DOTLINE, ESCAPED_DOTLINELEN);
+            result_ptr += ESCAPED_DOTLINELEN;
+            text += DOTLINELEN;
+        } else {
+            *(result_ptr++) = *(text++);
+        }
     }
 
-    /* Paste the rest */    
-    result=spd_string_append(result, (char*) &text[pos], -1);
-    
-    /*Escape the trailing '\r\n.' if found by adding an additional dot at the
-      end*/
-    if(strlen(result.str)>=3){
-	if(!strncmp(&result.str[strlen(result.str)-3], "\r\n.", 3)){
-	    result=spd_string_append(result, ".", -1);
-	    return result.str;
-	}
-    }
-
-    return result.str;
+    * result_ptr = '\0';
+    return result;
 }
 
 #ifdef LIBSPEECHD_DEBUG
