@@ -399,6 +399,8 @@ _generic_speak(void* nothing)
 		  play_command = strdup("play");
 		}else if (!strncmp(audio_settings.audio_output_method, "alsa", amlen)){
 		  play_command = strdup("aplay");
+		}else if (!strncmp(audio_settings.audio_output_method, "libao", amlen)){
+		  play_command = strdup("play");
 		}else if (!strncmp(audio_settings.audio_output_method, "pulse", amlen)){
 		  play_command = strdup("paplay");
 		}else{
@@ -506,12 +508,8 @@ _generic_child(TModuleDoublePipe dpipe, const size_t maxlen)
         /* Escape any quotes */
         message = g_string_new("");
         for(i=0; i<=bytes-1; i++){
-            if (text[i] == '\"')
-                message = g_string_append(message, "\\\"");
-            else if (text[i] == '`')
-                message = g_string_append(message, "\\`");
-            else if (text[i] == '\\')
-                message = g_string_append(message, "\\\\");
+            if (text[i] == '\'')
+                message = g_string_append(message, "'\\''");
 	    else{
                 g_string_append_printf(message, "%c", text[i]);
             }
