@@ -253,6 +253,7 @@ static void ibmtts_set_language_and_voice(char *lang, EVoiceType voice, char* di
 static void ibmtts_set_synthesis_voice(char *);
 static void ibmtts_set_rate(signed int rate);
 static void ibmtts_set_pitch(signed int pitch);
+static void ibmtts_set_punctuation_mode(EPunctMode punct_mode);
 static void ibmtts_set_volume(signed int pitch);
 
 /* Internal function prototypes for synthesis thread. */
@@ -299,9 +300,8 @@ static void* _ibmtts_stop_or_pause(void*);
 MOD_OPTION_1_INT(IbmttsMaxChunkLength);
 MOD_OPTION_1_STR(IbmttsDelimiters);
 
-/* Does IBM TTS support SSML or should we strip it off? */
 MOD_OPTION_1_INT(IbmttsUseSSML);
-
+MOD_OPTION_1_STR(IbmttsPunctuationList);
 MOD_OPTION_1_INT(IbmttsUseAbbreviation);
 MOD_OPTION_1_INT(IbmttsAudioChunkSize);
 MOD_OPTION_1_STR(IbmttsSoundIconFolder);
@@ -375,8 +375,9 @@ module_load(void)
     MOD_OPTION_1_INT_REG(IbmttsMaxChunkLength, 3000);
     MOD_OPTION_1_STR_REG(IbmttsDelimiters, "");
 	
-    MOD_OPTION_1_INT_REG(IbmttsUseSSML, 0);
+    MOD_OPTION_1_INT_REG(IbmttsUseSSML, 1);
     MOD_OPTION_1_INT_REG(IbmttsUseAbbreviation, 1);
+    MOD_OPTION_1_STR_REG(IbmttsPunctuationList, "()?");
 
     MOD_OPTION_1_INT_REG(IbmttsAudioChunkSize, 20000);
     MOD_OPTION_1_STR_REG(IbmttsSoundIconFolder, "/usr/share/sounds/sound-icons/");
@@ -459,6 +460,17 @@ module_init(char **status_info)
 
     eciSetParam (eciHandle, eciDictionary, !IbmttsUseAbbreviation);
 
+    /* enable annotations */
+    eciSetParam (eciHandle, eciInputType, 1);
+
+    /* load possibly the ssml filter */
+    eciAddText(eciHandle, " `gfa1 ");
+
+    /* load possibly the punctuation filter */
+    eciAddText(eciHandle, " `gfa2 ");
+
+    ibmtts_set_punctuation_mode(msg_settings.punctuation_mode);
+
     alloc_voice_list();
 
     /* These mutexes are locked when the corresponding threads are suspended. */
@@ -520,6 +532,7 @@ module_init(char **status_info)
     module_audio_id = NULL;
 
     *status_info = strdup("Ibmtts: Initialized successfully.");
+    g_string_free(info, 1);
 
     return OK;
 }
@@ -583,8 +596,9 @@ module_speak(gchar *data, size_t bytes, EMessageType msgtype)
     UPDATE_PARAMETER(rate, ibmtts_set_rate);
     UPDATE_PARAMETER(volume, ibmtts_set_volume);
     UPDATE_PARAMETER(pitch, ibmtts_set_pitch);
+    UPDATE_PARAMETER(punctuation_mode, ibmtts_set_punctuation_mode);
+
     /* TODO: Handle these in _ibmtts_synth() ?
-    UPDATE_PARAMETER(punctuation_mode, festival_set_punctuation_mode);
     UPDATE_PARAMETER(cap_let_recogn, festival_set_cap_let_recogn);
     */
 
@@ -832,6 +846,68 @@ _ibmtts_stop_or_pause(void* nothing)
     pthread_exit(NULL);
 }
 
+static int 
+process_text_mark(char *part, int part_len, char *mark_name)
+{
+    /* Handle index marks. */
+    if (NULL != mark_name) {
+        /* Assign the mark name an integer number and store in lookup table. */
+        int *markId = (int *) xmalloc( sizeof (int) );
+        *markId = 1 + g_hash_table_size(ibmtts_index_mark_ht);
+        g_hash_table_insert(ibmtts_index_mark_ht, markId, mark_name);
+        if (!eciInsertIndex(eciHandle, *markId)) {
+            DBG("Ibmtts: Error sending index mark to synthesizer.");
+            ibmtts_log_eci_error();
+            /* Try to keep going. */
+        } else
+            DBG("Ibmtts: Index mark |%s| (id %i) sent to synthesizer.",mark_name, *markId);
+        /* If pause is requested, skip over rest of message,
+           but synthesize what we have so far. */
+        if (ibmtts_pause_requested) {
+            DBG("Ibmtts: Pause requested in synthesis thread.");
+            return 1;
+        }
+        return 0;
+    }
+    
+    /* Handle normal text. */
+    if (part_len > 0) {
+        DBG("Ibmtts: Returned %d bytes from get_part.", part_len);
+        DBG("Ibmtts: Text to synthesize is |%s|\n", part);
+        DBG("Ibmtts: Sending text to synthesizer.");
+        if (!eciAddText(eciHandle, part)) {
+            DBG("Ibmtts: Error sending text.");
+            ibmtts_log_eci_error();
+            return 2;
+        }
+        return 0;
+    }
+    
+    /* Handle end of text. */
+    DBG("Ibmtts: End of data in synthesis thread.");
+    /*
+     Add index mark for end of message.
+     This also makes sure the callback gets called at least once 
+    */
+    eciInsertIndex(eciHandle, IBMTTS_MSG_END_MARK);
+    DBG("Ibmtts: Trying to synthesize text.");
+    if (!eciSynthesize(eciHandle)) {
+        DBG("Ibmtts: Error synthesizing.");
+        ibmtts_log_eci_error();
+        return 2;;
+    }
+
+    /* Audio and index marks are returned in eciCallback(). */
+    DBG("Ibmtts: Waiting for synthesis to complete.");
+    if (!eciSynchronize(eciHandle)) {
+        DBG("Ibmtts: Error waiting for synthesis to complete.");
+        ibmtts_log_eci_error();
+        return 2;
+    }
+    DBG("Ibmtts: Synthesis complete.");
+    return 3;
+    }
+
 /* Synthesis thread. */
 static void*
 _ibmtts_synth(void* nothing)
@@ -839,7 +915,7 @@ _ibmtts_synth(void* nothing)
     char *pos = NULL;
     char *part = NULL;
     int part_len = 0;
-    int *markId = NULL;
+    int ret;
     
     DBG("Ibmtts: Synthesis thread starting.......\n");
 
@@ -847,8 +923,7 @@ _ibmtts_synth(void* nothing)
     set_speaking_thread_parameters();
 
     /* Allocate a place for index mark names to be placed. */
-    char **mark_name = (char **) xmalloc(sizeof (char *));
-    *mark_name = NULL;
+    char *mark_name = NULL;
 
     while (!ibmtts_thread_exit_requested)
     {
@@ -919,10 +994,6 @@ _ibmtts_synth(void* nothing)
             }
 
             /* TODO: How to map these msg_settings to ibm tts?
-                EPunctMode punctuation_mode;
-                    PUNCT_NONE = 0,
-                    PUNCT_ALL = 1,
-                    PUNCT_SOME = 2
                 ESpellMode spelling_mode;
                     SPELLING_ON already handled in module_speak()
                 ECapLetRecogn cap_let_recogn;
@@ -931,87 +1002,23 @@ _ibmtts_synth(void* nothing)
                     RECOGN_ICON = 2
             */
 
-            part = ibmtts_next_part(pos, mark_name);
+            part = ibmtts_next_part(pos, &mark_name);
             if (NULL == part) {
                 DBG("Ibmtts: Error getting next part of message.");
                 /* TODO: What to do here? */
                 break;
-            } else {
-                part_len = strlen(part);
-                pos += part_len;
-            }
-
-            /* Handle index marks. */
-            if (NULL != *mark_name) {
-                /* Assign the mark name an integer number and store in lookup table. */
-                markId = (int *) xmalloc( sizeof (int) );
-                *markId = 1 + g_hash_table_size(ibmtts_index_mark_ht);
-                g_hash_table_insert(ibmtts_index_mark_ht, markId, strdup(*mark_name));
-                if (!eciInsertIndex(eciHandle, *markId)) {
-                    DBG("Ibmtts: Error sending index mark to synthesizer.");
-                    ibmtts_log_eci_error();
-                    /* Try to keep going. */
-                } else
-                    DBG("Ibmtts: Index mark |%s| (id %i) sent to synthesizer.",*mark_name, *markId);
-                xfree(*mark_name);
-                *mark_name = NULL;
-                /* If pause is requested, skip over rest of message,
-                   but synthesize what we have so far. */
-                if (ibmtts_pause_requested) {
-                    DBG("Ibmtts: Pause requested in synthesis thread.");
-                    pos += strlen(pos);
-                }
-            }
-            /* Handle normal text. */
-            else if (part_len > 0) {
-                DBG("Ibmtts: Returned %d bytes from get_part.", part_len);
-                DBG("Ibmtts: Text to synthesize is |%s|\n", part);
-                DBG("Ibmtts: Sending text to synthesizer.");
-                if (!eciAddText(eciHandle, part)) {
-                    DBG("Ibmtts: Error sending text.");
-                    ibmtts_log_eci_error();
-                    break;
-                }
-                xfree(part);
-                part = NULL;
-            }
-            /* Handle end of text. */
-            else {
-                xfree(part);
-                part = NULL;
-                DBG("Ibmtts: End of data in synthesis thread.");
-				/* Add index mark for end of message.
-				   This also makes sure the callback gets called at least once */
-				eciInsertIndex(eciHandle, IBMTTS_MSG_END_MARK);
-                DBG("Ibmtts: Trying to synthesize text.");
-                if (!eciSynthesize(eciHandle)) {
-                    DBG("Ibmtts: Error synthesizing.");
-                    ibmtts_log_eci_error();
-                    break;
-                }
-				
-                /* Audio and index marks are returned in eciCallback(). */
-                DBG("Ibmtts: Waiting for synthesis to complete.");
-                if (!eciSynchronize(eciHandle)) {
-                    DBG("Ibmtts: Error waiting for synthesis to complete.");
-                    ibmtts_log_eci_error();
-                    break;
-                }
-				DBG("Ibmtts: Synthesis complete.");
-                                break;
-            }
-
-            if (ibmtts_stop_synth_requested){
-                DBG("Ibmtts: Stop in synthesis thread, terminating.");
-                break;
             }
+            part_len = strlen(part);
+            pos += part_len;
+            ret = process_text_mark(part, part_len, mark_name);
+            free(part);
+            part = NULL;
+            mark_name = NULL;
+            if (ret == 1) pos += strlen(pos);
+            else if (ret > 1) break;
         }
     }
 
-    xfree(part);
-    xfree(*mark_name);
-    xfree(mark_name);
-
     DBG("Ibmtts: Synthesis thread ended.......\n");
 
     pthread_exit(NULL);
@@ -1095,6 +1102,19 @@ ibmtts_set_pitch(signed int pitch)
         DBG("Ibmtts: Pitch set to %i.", pitchBaseline);
 }
 
+static void
+ibmtts_set_punctuation_mode(EPunctMode punct_mode)
+{
+    const char* fmt = "`Pf%d%s";
+    size_t len = strlen(fmt) + strlen(IbmttsPunctuationList) + sizeof('\0');
+    char* msg = malloc(len);
+
+    if (msg) {
+        snprintf(msg, len, fmt, punct_mode, IbmttsPunctuationList);
+        eciAddText(eciHandle, msg);
+        free(msg);
+    }
+}
 
 static char*
 ibmtts_voice_enum_to_str(EVoiceType voice)
@@ -1432,19 +1452,19 @@ _ibmtts_play(void* nothing)
             pthread_mutex_lock(&ibmtts_play_suspended_mutex);
             sem_wait(ibmtts_play_semaphore);
             pthread_mutex_unlock(&ibmtts_play_suspended_mutex);
-            if (ibmtts_thread_exit_requested) break;
         }
         /* DBG("Ibmtts: Playback semaphore on."); */
 
-        pthread_mutex_lock(&playback_queue_mutex);
-        if (NULL != playback_queue) {
-            playback_queue_entry = playback_queue->data;
-            playback_queue = g_slist_remove(playback_queue, playback_queue->data);
-        }
-        pthread_mutex_unlock(&playback_queue_mutex);
-
-        while ((NULL != playback_queue_entry) && !ibmtts_stop_play_requested)
+        while (!ibmtts_stop_play_requested && !ibmtts_thread_exit_requested)
         {
+            pthread_mutex_lock(&playback_queue_mutex);
+            if (NULL != playback_queue) {
+                playback_queue_entry = playback_queue->data;
+                playback_queue = g_slist_remove(playback_queue, playback_queue->data);
+            }
+            pthread_mutex_unlock(&playback_queue_mutex);
+            if (NULL == playback_queue_entry) break;
+
             switch (playback_queue_entry->type) {
                 case IBMTTS_QET_AUDIO:
                     ibmtts_send_to_audio(playback_queue_entry);
@@ -1483,14 +1503,7 @@ _ibmtts_play(void* nothing)
 
             ibmtts_delete_playback_queue_entry(playback_queue_entry);
             playback_queue_entry = NULL;
-
-            pthread_mutex_lock(&playback_queue_mutex);
-            if (NULL != playback_queue) {
-                playback_queue_entry = playback_queue->data;
-                playback_queue = g_slist_remove(playback_queue, playback_queue->data);
-            }
-            pthread_mutex_unlock(&playback_queue_mutex);
-		}
+        }
 		if (ibmtts_stop_play_requested) DBG("Ibmtts: Stop or pause in playback thread.");
 	}
 
