@@ -1,7 +1,7 @@
 /*
  * module_utils_addvoice.c - Functionality for the DotConf AddVoice feature
  * 
- * Copyright (C) 2001, 2002, 2003, 2006 Brailcom, o.p.s.
+ * Copyright (C) 2001, 2002, 2003, 2006, 2007 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -18,7 +18,7 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  *
- * $Id: module_utils_addvoice.c,v 1.4 2006/07/11 16:12:27 hanke Exp $
+ * $Id: module_utils_addvoice.c,v 1.5 2007-07-29 23:43:33 hanke Exp $
  */
 
 #include <glib.h>
@@ -96,8 +96,8 @@ DOTCONF_CB(AddVoice_cb)
     return NULL;
 }
 
-static void
-module_register_settings_voices()
+void
+module_register_settings_voices(void)
 {
     module_voice_table = g_hash_table_new(g_str_hash, g_str_equal);
     module_dc_options = module_add_config_option(module_dc_options,
@@ -105,7 +105,7 @@ module_register_settings_voices()
                                                  ARG_LIST, AddVoice_cb, NULL, 0);
 }
 
-static char*
+char*
 module_getvoice(char* language, EVoiceType voice)
 {
     SPDVoiceDef *voices;
@@ -141,7 +141,7 @@ module_getvoice(char* language, EVoiceType voice)
         ret = voices->child_female; break;
     default:
         printf("Unknown voice");
-        exit(1);
+        return NULL;
     }
 
     if (ret == NULL) ret = voices->male1;
