@@ -62,7 +62,50 @@ class SSIPError(Exception):
     
 class SSIPCommunicationError(SSIPError):
     """Exception raised when trying to operate on a closed connection."""
-    
+
+    _additional_exception = None
+
+    def __init__(self, description=None, original_exception=None, **kwargs):
+        self._original_exception = original_exception
+        self._description = description
+        super(SSIPError, self).__init__(**kwargs)
+
+    def original_exception(self):
+        """Return the original exception if any
+
+        If this exception is secondary, being caused by a lower
+        level exception, return this original exception, otherwise
+        None"""
+        return self._original_exception
+
+    def set_additional_exception(self, exception):
+        """Set an additional exception
+        
+        See method additional_exception().
+        """
+        self._additional_exception = exception
+
+    def additional_exception(self):
+        """Return an additional exception
+        
+        Additional exceptions araise from failed attempts to resolve
+        the former problem"""
+        return self._additional_exception
+
+    def description(self):
+        """Return error description"""
+        return self._description
+
+    def __str__(self):
+        msgs = []
+        if self.description():
+            msgs.append(self.description())
+        if self.original_exception:
+            msgs.append("Original error: " + str(self.original_exception()))
+        if self.additional_exception:
+            msgs.append("Additional error: " + str(self.additional_exception()))
+        return "\n".join(msgs)
+
 class SSIPResponseError(Exception):
     def __init__(self, code, msg, data):
         Exception.__init__(self, "%s: %s" % (code, msg))
@@ -98,7 +141,7 @@ class SSIPDataError(SSIPResponseError):
 class SpawnError(Exception):
     """Indicates failure in server autospawn."""
 
-class ConnectionMethod(object):
+class CommunicationMethod(object):
     """Constants describing the possible methods of connection to server."""
     UNIX_SOCKET = 'unix_socket'
     """Unix socket communication using a filesystem path"""
@@ -113,6 +156,9 @@ class _SSIP_Connection(object):
     _END_OF_DATA_MARKER_ESCAPED = '..'
     _END_OF_DATA = _NEWLINE + _END_OF_DATA_MARKER + _NEWLINE
     _END_OF_DATA_ESCAPED = _NEWLINE + _END_OF_DATA_MARKER_ESCAPED + _NEWLINE
+    # Constants representing \r\n. and \r\n..
+    _RAW_DOTLINE = _NEWLINE + _END_OF_DATA_MARKER
+    _ESCAPED_DOTLINE = _NEWLINE + _END_OF_DATA_MARKER_ESCAPED
 
     _CALLBACK_TYPE_MAP = {700: CallbackType.INDEX_MARK,
                           701: CallbackType.BEGIN,
@@ -122,16 +168,16 @@ class _SSIP_Connection(object):
                           705: CallbackType.RESUME,
                           }
 
-    def __init__(self, method, socket_path, host, port):
+    def __init__(self, communication_method, socket_path, host, port):
         """Init connection: open the socket to server,
         initialize buffers, launch a communication handling
         thread.
         """
 
-        if method == ConnectionMethod.UNIX_SOCKET:
+        if communication_method == CommunicationMethod.UNIX_SOCKET:
             socket_family = socket.AF_UNIX
-            socket_connect_args = (socket_path,)
-        elif method == ConnectionMethod.INET_SOCKET:
+            socket_connect_args = socket_path
+        elif communication_method == CommunicationMethod.INET_SOCKET:
             assert host and port
             socket_family = socket.AF_INET
             socket_connect_args = (socket.gethostbyname(host), port)
@@ -140,9 +186,11 @@ class _SSIP_Connection(object):
 
         try:
             self._socket = socket.socket(socket_family, socket.SOCK_STREAM)
-            self._socket.connect(*socket_connect_args)
-        except socket.error:
-            raise SSIPCommunicationError("Can't open socket using method" + method)
+            self._socket.connect(socket_connect_args)
+        except socket.error, ex:
+            raise SSIPCommunicationError("Can't open socket using method "
+                                         + communication_method,
+                                         original_exception = ex)
 
         self._buffer = ""
         self._com_buffer = []
@@ -293,12 +341,17 @@ class _SSIP_Connection(object):
         
         """
         # Escape the end-of-data marker even if present at the beginning
-        if data.startswith(self._END_OF_DATA_MARKER + self._NEWLINE):
+        # The start of the string is also the start of a line.
+        if data.startswith(self._END_OF_DATA_MARKER):
             l = len(self._END_OF_DATA_MARKER)
             data = self._END_OF_DATA_MARKER_ESCAPED + data[l:]
-        elif data == self._END_OF_DATA_MARKER:
-            data = self._END_OF_DATA_MARKER_ESCAPED
-        data = data.replace(self._END_OF_DATA, self._END_OF_DATA_ESCAPED)
+
+        # Escape the end of data marker at the start of each subsequent
+        # line.  We can do that by simply replacing \r\n. with \r\n..,
+        # since the start of a line is immediately preceded by \r\n,
+        # when the line is not the beginning of the string.
+        data = data.replace(self._RAW_DOTLINE, self._ESCAPED_DOTLINE)
+
         try:
             self._socket.send(data + self._END_OF_DATA)
         except socket.error:
@@ -330,6 +383,39 @@ class _SSIP_Connection(object):
         """
         self._callback = callback
 
+class _CallbackHandler(object):
+    """Internal object which handles callbacks."""
+
+    def __init__(self, client_id):
+        self._client_id = client_id
+        self._callbacks = {}
+        self._lock = threading.Lock()
+
+    def __call__(self, msg_id, client_id, type, **kwargs):
+        if client_id != self._client_id:
+            # TODO: does that ever happen?
+            return
+        self._lock.acquire()
+        try:
+            try:
+                callback, event_types = self._callbacks[msg_id]
+            except KeyError:
+                pass
+            else:
+                if event_types is None or type in event_types:
+                    callback(type, **kwargs)
+                if type in (CallbackType.END, CallbackType.CANCEL):
+                    del self._callbacks[msg_id]
+        finally:
+            self._lock.release()
+
+    def add_callback(self, msg_id,  callback, event_types):
+        self._lock.acquire()
+        try:
+            self._callbacks[msg_id] = (callback, event_types)
+        finally:
+            self._lock.release()
+
 class Scope(object):
     """An enumeration of valid SSIP command scopes.
 
@@ -375,6 +461,17 @@ class PunctuationMode(object):
 
     """
 
+class DataMode(object):
+    """Constants specifying the type of data contained within messages
+    to be spoken.
+
+    """
+    TEXT = 'text'
+    """Data is plain text."""
+    SSML = 'ssml'
+    """Data is SSML (Speech Synthesis Markup Language)."""
+
+
 class SSIPClient(object):
     """Basic Speech Dispatcher client interface.
 
@@ -399,9 +496,10 @@ class SSIPClient(object):
     DEFAULT_SOCKET_PATH = "~/.speech-dispatcher/speechd.sock"
     """Default name of the communication unix socket"""
     
-    def __init__(self, name, component='default', user='unknown', host=None,
-                 port=None, method=ConnectionMethod.UNIX_SOCKET, socket_path=None,
-                 autospawn=None):
+    def __init__(self, name, component='default', user='unknown', address=None,
+                 autospawn=None,
+                 # Deprecated ->
+                 host=None, port=None, method=None, socket_path=None):
         """Initialize the instance and connect to the server.
 
         Arguments:
@@ -410,100 +508,171 @@ class SSIPClient(object):
             multiple connections, this can be used to identify each of them.
           user -- user identification string (user name).  When multi-user
             acces is expected, this can be used to identify their connections.
-          method -- communication method to use, one of the constants defined in class
-            ConnectionMethod
-          socket_path -- for ConnectionMethod.UNIX_SOCKET, socket
-            path in filesystem. By default, this is ~/.speech-dispatcher/speechd.sock
-            where `~' is the users home directory as determined from the system
-            defaults and from the $HOMEDIR environment variable.
-          host -- for ConnectionMethod.INET_SOCKET, server hostname
-            or IP address as a string.  If None, the default value is
-            taken from SPEECHD_HOST environment variable (if it
-            exists) or from the DEFAULT_HOST attribute of this class.
-          port -- for ConnectionMethod.INET_SOCKET method, server
-            port as number or None.  If None, the default value is
-            taken from SPEECHD_PORT environment variable (if it
-            exists) or from the DEFAULT_PORT attribute of this class.
+          address -- server address as specified in Speech Dispatcher
+            documentation (e.g. "unix:/home/joe/.speech-dispatcher/speechd.sock"
+            or "inet:192.168.0.85:6561")
           autospawn -- a flag to specify whether the library should
             try to start the server if it determines its not already
             running or not
-        
+
+        Deprecated arguments:
+          method -- communication method to use, one of the constants defined in class
+            CommunicationMethod
+          socket_path -- for CommunicationMethod.UNIX_SOCKET, socket
+            path in filesystem. By default, this is ~/.speech-dispatcher/speechd.sock
+            where `~' is the users home directory as determined from the system
+            defaults and from the $HOMEDIR environment variable.
+          host -- for CommunicationMethod.INET_SOCKET, server hostname
+            or IP address as a string.  If None, the default value is
+            taken from SPEECHD_HOST environment variable (if it
+            exists) or from the DEFAULT_HOST attribute of this class.
+          port -- for CommunicationMethod.INET_SOCKET method, server
+            port as number or None.  If None, the default value is
+            taken from SPEECHD_PORT environment variable (if it
+            exists) or from the DEFAULT_PORT attribute of this class.
+         
         For more information on client identification strings see Speech
         Dispatcher documentation.
-          
         """
-        if socket_path is None:
-            socket_path = os.environ.get('SPEECHD_SOCKET',
-                                         os.path.expanduser(self.DEFAULT_SOCKET_PATH))
-        if host is None:
-            host = os.environ.get('SPEECHD_HOST', self.DEFAULT_HOST)
-        if port is None:
-            try:
-                port = int(os.environ.get('SPEECHD_PORT'))
-            except (ValueError, TypeError):
-                port = self.DEFAULT_PORT
 
-        connection_args = {'method': method, 'socket_path': socket_path,
-                           'host': host, 'port': port}
+        # Resolve connection parameters:
+        connection_args = {'communication_method': CommunicationMethod.UNIX_SOCKET,
+                           'socket_path': os.path.expanduser(self.DEFAULT_SOCKET_PATH),
+                           'host': self.DEFAULT_HOST,
+                           'port': self.DEFAULT_PORT,
+                           }
+        # Respect address method argument and SPEECHD_ADDRESS environemt variable
+        _address = address or os.environ.get("SPEECHD_ADDRESS")        
+
+        if _address:
+            connection_args.update(self._connection_arguments_from_address(_address))
+        # Respect the old (deprecated) key arguments and environment variables
+        # TODO: Remove this section in 0.8 release
+        else:
+            # Read the environment variables
+            env_speechd_host = os.environ.get("SPEECHD_HOST")
+            try:
+                env_speechd_port = int(os.environ.get("SPEECHD_PORT"))
+            except:
+                env_speechd_port = None
+            env_speechd_socket_path = os.environ.get("SPEECHD_SOCKET")
+            # Prefer old (deprecated) function arguments, but if
+            # not specified and old (deprecated) environment variable
+            # is set, use the value of the environment variable
+            if method:
+                connection_args['method'] = method
+            if port:
+                connection_args['port'] = port
+            elif env_speechd_port:
+                connection_args['port'] = env_speechd_port
+            if socket_path:
+                connection_args['socket_path'] = socket_path
+            elif env_speechd_socket_path:
+                connection_args['socket_path'] = env_speechd_socket_path
+        self._connect_with_autospawn(connection_args, autospawn)
+        self._initialize_connection(user, name, component)
+
+    def _connect_with_autospawn(self, connection_args, autospawn):
+        """Establish new connection (and/or autospawn server)"""
         try:
-            self._conn = conn = _SSIP_Connection(**connection_args)
-        except SSIPCommunicationError:
+            self._conn = _SSIP_Connection(**connection_args)
+        except SSIPCommunicationError, ce:
             # Suppose server might not be running, try the autospawn mechanism
             if autospawn != False:
                 # Autospawn is however not guaranteed to start the server. The server
                 # will decide, based on it's configuration, whether to honor the request.
-                self._server_spawn()
-                self._conn = conn = _SSIP_Connection(**connection_args)
+                try:
+                    self._server_spawn(connection_args)
+                except SpawnError, se:
+                    ce.set_additional_exception(se)
+                    raise ce
+                self._conn = _SSIP_Connection(**connection_args)
             else:
                 raise
 
+    def _initialize_connection(self, user, name, component):
+        """Initialize connection -- Set client name, get id, register callbacks etc."""
         full_name = '%s:%s:%s' % (user, name, component)
-        conn.send_command('SET', Scope.SELF, 'CLIENT_NAME', full_name)
-        code, msg, data = conn.send_command('HISTORY', 'GET', 'CLIENT_ID')
+        self._conn.send_command('SET', Scope.SELF, 'CLIENT_NAME', full_name)
+        code, msg, data = self._conn.send_command('HISTORY', 'GET', 'CLIENT_ID')
         self._client_id = int(data[0])
-        self._lock = threading.Lock()
-        self._callbacks = {}
-        conn.set_callback(self._callback_handler)
+        self._callback_handler = _CallbackHandler(self._client_id)
+        self._conn.set_callback(self._callback_handler)
         for event in (CallbackType.INDEX_MARK,
                       CallbackType.BEGIN,
                       CallbackType.END,
                       CallbackType.CANCEL,
                       CallbackType.PAUSE,
                       CallbackType.RESUME):
-            conn.send_command('SET', 'self', 'NOTIFICATION', event, 'on')
+            self._conn.send_command('SET', 'self', 'NOTIFICATION', event, 'on')
 
+    def _connection_arguments_from_address(self, address):
+        """Parse a Speech Dispatcher address line and return a dictionary
+        of connection arguments"""
+        connection_args = {}
+        address_params = address.split(":")
+        try:
+            _method = address_params[0]
+        except:
+            raise SSIPCommunicationErrror("Wrong format of server address")
+        connection_args['communication_method'] = _method
+        if _method == CommunicationMethod.UNIX_SOCKET:
+            try:
+                connection_args['socket_path'] = address_params[1]
+            except IndexError:
+                pass # The additional parameters was not set, let's stay with defaults
+        elif _method == CommunicationMethod.INET_SOCKET:
+            try:
+                connection_args['host'] = address_params[1]
+                connection_args['port'] = int(address_params[2])
+            except ValueError: # Failed conversion to int
+                raise SSIPCommunicationError("Third parameter of inet_socket address must be a port number")
+            except IndexError:
+                pass # The additional parameters was not set, let's stay with defaults
+        else:
+            raise SSIPCommunicationError("Unknown communication method in address.");
+        return connection_args
     
     def __del__(self):
         """Close the connection"""
         self.close()
 
-    def _callback_handler(self, msg_id, client_id, type, **kwargs):
-        if client_id != self._client_id:
-            # TODO: does that ever happen?
-            return
-        self._lock.acquire()
-        try:
-            try:
-                callback, event_types = self._callbacks[msg_id]
-            except KeyError:
-                pass
-            else:
-                if event_types is None or type in event_types:
-                    callback(type, **kwargs)
-                if type in (CallbackType.END, CallbackType.CANCEL):
-                    del self._callbacks[msg_id]
-        finally:
-            self._lock.release()
-
-    def _server_spawn(self):
+    def _server_spawn(self, connection_args):
         """Attempts to spawn the speech-dispatcher server."""
+        # Check whether we are not connecting to a remote host
+        # TODO: This is a hack. inet sockets specific code should
+        # belong to _SSIPConnection. We do not however have an _SSIPConnection
+        # yet.
+        if connection_args['communication_method'] == 'inet_socket':
+            addrinfos = socket.getaddrinfo(connection_args['host'],
+                                           connection_args['port'])
+            # Check resolved addrinfos for presence of localhost
+            ip_addresses = [addrinfo[4][0] for addrinfo in addrinfos]
+            localhost=False
+            for ip in ip_addresses:
+                if ip.startswith("127.") or ip == "::1":
+                    connection_args['host'] = ip
+                    localhost=True
+            if not localhost:
+                # The hostname didn't resolve on localhost in neither case,
+                # do not spawn server on localhost...
+                raise SpawnError(
+                    "Can't start server automatically (autospawn), requested address %s "
+                    "resolves on %s which seems to be a remote host. You must start the "
+                    "server manually or choose another connection address." % (connection_args['host'],
+                                                                               str(ip_addresses),))
         if os.path.exists(paths.SPD_SPAWN_CMD):
-            server = subprocess.Popen([paths.SPD_SPAWN_CMD, "--spawn"], stdin=None,
-                                      stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-            #TODO: When logging will be implemented in this library, it will be very
-            #desirable to read the stdout output and log it because it contains valuable
-            #information on the result of the autospawn and the reason for it
-            server.wait()
+            connection_params = []
+            for param, value in connection_args.items():
+                if param not in ["host",]:
+                    connection_params += ["--"+param.replace("_","-"), str(value)]
+
+            server = subprocess.Popen([paths.SPD_SPAWN_CMD, "--spawn"]+connection_params,
+                                      stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            stdout_reply, stderr_reply = server.communicate()
+            retcode = server.wait()
+            if retcode != 0:
+                raise SpawnError("Server refused to autospawn, stating this reason: %s" % (stderr_reply,))
             return server.pid
         else:
             raise SpawnError("Can't find Speech Dispatcher spawn command %s"
@@ -521,6 +690,23 @@ class SSIPClient(object):
                             Priority.PROGRESS), priority
         self._conn.send_command('SET', Scope.SELF, 'PRIORITY', priority)
 
+    def set_data_mode(self, value):
+        """Set the data mode for further speech commands.
+
+        Arguments:
+          value - one of the constants defined by the DataMode class.
+
+        """
+        if value == DataMode.SSML:
+            ssip_val = 'on'
+        elif value == DataMode.TEXT:
+            ssip_val = 'off'
+        else:
+            raise ValueError(
+                'Value "%s" is not one of the constants from the DataMode class.' % \
+                    value)
+        self._conn.send_command('SET', Scope.SELF, 'SSML_MODE', ssip_val)
+
     def speak(self, text, callback=None, event_types=None):
         """Say given message.
 
@@ -557,13 +743,9 @@ class SSIPClient(object):
         if callback:
             msg_id = int(result[2][0])
             # TODO: Here we risk, that the callback arrives earlier, than we
-            # add the item to `self._callbacks'.  Such a situation will lead to
-            # the callback being ignored.
-            self._lock.acquire()
-            try:
-                self._callbacks[msg_id] = (callback, event_types)
-            finally:
-                self._lock.release()
+            # add the item to `self._callback_handler'.  Such a situation will
+            # lead to the callback being ignored.
+            self._callback_handler.add_callback(msg_id, callback, event_types)
         return result
 
     def char(self, char):
@@ -609,7 +791,7 @@ class SSIPClient(object):
         """Immediately stop speaking and discard messages in queues.
 
         Arguments:
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         self._conn.send_command('CANCEL', scope)
@@ -619,7 +801,7 @@ class SSIPClient(object):
         """Immediately stop speaking the currently spoken message.
 
         Arguments:
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
         
         """
         self._conn.send_command('STOP', scope)
@@ -632,7 +814,7 @@ class SSIPClient(object):
         sentence).
 
         Arguments:
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
         
         """
         self._conn.send_command('PAUSE', scope)
@@ -645,7 +827,7 @@ class SSIPClient(object):
         sentence).
 
         Arguments:
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
         
         """
         self._conn.send_command('RESUME', scope)
@@ -678,7 +860,7 @@ class SSIPClient(object):
 
         Arguments:
           language -- two letter language code according to RFC 1776 as string.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         assert isinstance(language, str) and len(language) == 2
@@ -689,7 +871,7 @@ class SSIPClient(object):
 
         Arguments:
           name -- module (string) as returned by 'list_output_modules()'.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
         
         """
         self._conn.send_command('SET', scope, 'OUTPUT_MODULE', name)
@@ -702,7 +884,7 @@ class SSIPClient(object):
             corresponding to the default pitch of the current speech synthesis
             output module, lower values meaning lower pitch and higher values
             meaning higher pitch.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
           
         """
         assert isinstance(value, int) and -100 <= value <= 100, value
@@ -716,7 +898,7 @@ class SSIPClient(object):
             corresponding to the default speech rate of the current speech
             synthesis output module, lower values meaning slower speech and
             higher values meaning faster speech.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         assert isinstance(value, int) and -100 <= value <= 100
@@ -729,7 +911,7 @@ class SSIPClient(object):
           value -- integer value within the range from -100 to 100, with 100
             corresponding to the default speech volume of the current speech
             synthesis output module, lower values meaning softer speech.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         assert isinstance(value, int) and -100 <= value <= 100
@@ -740,7 +922,7 @@ class SSIPClient(object):
 
         Arguments:
           value -- one of the 'PunctuationMode' constants.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         assert value in (PunctuationMode.ALL, PunctuationMode.SOME,
@@ -754,7 +936,7 @@ class SSIPClient(object):
           value -- if 'True', all incomming messages will be spelled
             instead of being read as normal words. 'False' switches
             this behavior off.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         assert value in [True, False]
@@ -771,7 +953,7 @@ class SSIPClient(object):
             of capital letters, 'spell' means capital letters will be spelled
             with a syntetic voice and 'icon' means that the capital-letter icon
             will be prepended before each capital letter.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         assert value in ("none", "spell", "icon")
@@ -783,7 +965,7 @@ class SSIPClient(object):
         Arguments:
           value -- one of the SSIP symbolic voice names: 'MALE1' .. 'MALE3',
             'FEMALE1' ... 'FEMALE3', 'CHILD_MALE', 'CHILD_FEMALE'
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
 
         Symbolic voice names are mapped to real synthesizer voices in the
         configuration of the output module.  Use the method
@@ -801,7 +983,7 @@ class SSIPClient(object):
 
         Arguments:
           value -- voice name as returned by 'list_synthesis_voices()'
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         self._conn.send_command('SET', scope, 'SYNTHESIS_VOICE', value)
@@ -812,7 +994,7 @@ class SSIPClient(object):
         Arguments:
           value -- a positive or negative value meaning how many chunks of data
             after or before the pause should be read when resume() is executed.
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
             
         """
         assert isinstance(value, int)
@@ -833,7 +1015,7 @@ class SSIPClient(object):
         Arguments:
           val -- a boolean value determining whether debugging
                  is switched on or off
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
         
         """
         assert isinstance(val, bool)
@@ -851,7 +1033,7 @@ class SSIPClient(object):
         Arguments:
           path -- path (string) to the directory where debuging
                   files will be created
-          scope -- see the documentaion of this class.
+          scope -- see the documentation of this class.
         
         """
         assert isinstance(val, string)
@@ -878,6 +1060,7 @@ class SSIPClient(object):
         """Close the connection to Speech Dispatcher."""
         if hasattr(self, '_conn'):
             self._conn.close()
+            del self._conn
 
 
 class Client(SSIPClient):
@@ -927,3 +1110,16 @@ class Speaker(SSIPClient):
     interface practical for screen readers.
     
     """
+
+
+# Deprecated but retained for backwards compatibility
+
+# This class was introduced in 0.7 but later renamed to CommunicationMethod
+class ConnectionMethod(object):
+    """Constants describing the possible methods of connection to server.
+
+    Retained for backwards compatibility but DEPRECATED. See CommunicationMethod."""
+    UNIX_SOCKET = 'unix_socket'
+    """Unix socket communication using a filesystem path"""
+    INET_SOCKET = 'inet_socket'
+    """Inet socket communication using a host and port"""
