@@ -60,6 +60,28 @@ speechd_queue_alloc()
 	return(new);
 }
 
+TFDSetElement
+spd_fdset_copy(TFDSetElement old)
+{
+    TFDSetElement new;
+
+    new = old;
+    new.language = spd_strdup(old.language);
+    new.synthesis_voice = spd_strdup(old.synthesis_voice);
+    new.client_name = spd_strdup(old.client_name);
+    new.output_module = spd_strdup(old.output_module);
+    new.index_mark = spd_strdup(old.index_mark);
+    new.audio_output_method = spd_strdup(old.audio_output_method);
+    new.audio_oss_device = spd_strdup(old.audio_oss_device);
+    new.audio_alsa_device = spd_strdup(old.audio_alsa_device);
+    new.audio_nas_server = spd_strdup(old.audio_nas_server);
+    new.audio_pulse_server = spd_strdup(old.audio_pulse_server);
+
+    return new;
+    
+    
+}
+
 TSpeechDMessage*
 spd_message_copy(TSpeechDMessage *old)
 {
@@ -70,17 +92,10 @@ spd_message_copy(TSpeechDMessage *old)
 	new = (TSpeechDMessage *) spd_malloc(sizeof(TSpeechDMessage));
 
 	*new = *old;
-
 	new->buf = spd_malloc((old->bytes+1) * sizeof(char));
 	memcpy(new->buf, old->buf, old->bytes);
-	new->buf[new->bytes] = 0;
-
-	new->settings = old->settings; 
-
-	new->settings.language = spd_strdup(old->settings.language);
-	new->settings.client_name = spd_strdup(old->settings.client_name);
-	new->settings.output_module = spd_strdup(old->settings.output_module);
-	new->settings.index_mark = spd_strdup(old->settings.index_mark);
+	new->buf[new->bytes] = 0;	
+	new->settings = spd_fdset_copy(old->settings); 
 
 	return new;
 }
@@ -92,8 +107,14 @@ mem_free_fdset(TFDSetElement *fdset)
        in a TSpeechDMessage */
     spd_free(fdset->client_name);
     spd_free(fdset->language);
+    spd_free(fdset->synthesis_voice);
     spd_free(fdset->output_module);
     spd_free(fdset->index_mark);
+    spd_free(fdset->audio_output_method);
+    spd_free(fdset->audio_oss_device);
+    spd_free(fdset->audio_alsa_device);
+    spd_free(fdset->audio_nas_server);
+    spd_free(fdset->audio_pulse_server);
 }
 
 void
