@@ -36,21 +36,21 @@ options_print_help(char *argv[])
     assert(argv);
     assert(argv[0]);
 
-    printf("Usage: %s [-{d|s}] [-l {1|2|3|4|5}] [-c com_method] [-S socket_name] [-p port] | [-v] | [-h]\n", argv[0]);
+    printf("Usage: %s [-{d|s}] [-l {1|2|3|4|5}] [-c com_method] [-S socket_path] [-p port] | [-v] | [-h]\n", argv[0]);
     printf("Speech Dispatcher -- Common interface for Speech Synthesis (GNU GPL)\n\n");
     printf("-d, --run-daemon     -      Run as a daemon\n"
     "-s, --run-single     -      Run as single application\n"
     "-a, --spawn          -      Start only if autospawn is not disabled\n"
     "-l, --log-level      -      Set log level (1..5)\n"
     "-c, --communication-method  Communication method to use (unix_socket or inet_socket)\n"
-    "-S, --socket-name    -      Socket name to use for 'unix_socket' method (filesystem path or 'default')\n"
+    "-S, --socket-path    -      Socket path to use for 'unix_socket' method (filesystem path or 'default')\n"
     "-p, --port           -      Specify a port number for 'inet_socket' method\n"
     "-P, --pid-file       -      Set path to pid file\n"
     "-C, --config-dir     -      Set path to configuration\n"
     "-v, --version        -      Report version of this program\n"
     "-D, --debug          -      Output debugging information into /tmp/.speech-dispatcher\n"
     "-h, --help           -      Print this info\n\n"
-    "Copyright (C) 2003,2006 Brailcom, o.p.s.\n"
+    "Copyright (C) 2003, 2004, 2006, 2007, 2008, 2010 Brailcom, o.p.s.\n"
     "This is free software; you can redistribute it and/or modify it\n"
     "under the terms of the GNU General Public License as published by\n"
     "the Free Software Foundation; either version 2, or (at your option)\n"
@@ -63,7 +63,7 @@ void
 options_print_version(void)
 {
     printf("%s %s\n", PACKAGE, VERSION);
-    printf("Copyright (C) 2002, 2003, 2006 Brailcom, o.p.s.\n"
+    printf("Copyright (C) 2003, 2004, 2006, 2007, 2008, 2010 Brailcom, o.p.s.\n"
            "Speech Dispatcher comes with ABSOLUTELY NO WARRANTY.\n"
            "You may redistribute copies of Speech Dispatcher\n"
            "under the terms of the GNU General Public License.\n"
@@ -79,7 +79,7 @@ options_print_version(void)
     }
 
 #define SPD_OPTION_SET_STR(param) \
-    SpeechdOptions.param = optarg
+    SpeechdOptions.param = strdup(optarg)
 
 void
 options_parse(int argc, char *argv[])
@@ -115,9 +115,11 @@ options_parse(int argc, char *argv[])
             break;
         case 'c':
             SPD_OPTION_SET_STR(communication_method);
+            SpeechdOptions.communication_method_set = 1;
             break;
         case 'S':
-            SPD_OPTION_SET_STR(socket_name);
+            SPD_OPTION_SET_STR(socket_path);
+            SpeechdOptions.socket_path_set = 1;
             break;
         case 'p':
             SPD_OPTION_SET_INT(port);
