@@ -46,13 +46,13 @@
 #define CHECK_SSIP_COMMAND(cmd_name, parse_function, allowed_in_block)\
     if(!strcmp(command, cmd_name)){ \
         spd_free(command); \
-        if ((allowed_in_block == BLOCK_NO) && SpeechdSocket[fd].inside_block) \
+        if ((allowed_in_block == BLOCK_NO) && speechd_socket->inside_block) \
             return strdup(ERR_NOT_ALLOWED_INSIDE_BLOCK); \
-        return (char*) (parse_function) (buf, bytes, fd); \
+        return (char*) (parse_function) (buf, bytes, fd, speechd_socket);	\
     }
 
 #define NOT_ALLOWED_INSIDE_BLOCK() \
-    if(SpeechdSocket[fd].inside_block > 0) \
+    if(speechd_socket->inside_block > 0) \
         return strdup(ERR_NOT_ALLOWED_INSIDE_BLOCK);
 
 #define ALLOWED_INSIDE_BLOCK() ;
@@ -69,10 +69,10 @@ parse(const char *buf, const int bytes, const int fd)
     int msg_uid;
     GString* ok_queued_reply;
     char *reply;
+    TSpeechDSock *speechd_socket = speechd_socket_get_by_fd(fd);
+    assert(speechd_socket);       
 
     end_data = 0;
-	
-    assert(fd > 0);
     if ((buf == NULL) || (bytes == 0)){
         if(SPEECHD_DEBUG) FATAL("invalid buffer for parse()\n");
         return strdup(ERR_INTERNAL);
@@ -80,7 +80,7 @@ parse(const char *buf, const int bytes, const int fd)
  
     /* First the condition that we are not in data mode and we
      * are awaiting commands */
-    if (SpeechdSocket[fd].awaiting_data == 0){
+    if (speechd_socket->awaiting_data == 0){
         /* Read the command */
         command = get_param(buf, 0, bytes, 1);
 
@@ -111,7 +111,10 @@ parse(const char *buf, const int bytes, const int fd)
         if (!strcmp(command,"bye") || !strcmp(command,"quit")){
             MSG(4, "Bye received.");
             /* Send a reply to the socket */
-            write(fd, OK_BYE, strlen(OK_BYE));
+            if (write(fd, OK_BYE, strlen(OK_BYE))){
+		MSG(2, "ERROR: Can't write OK_BYE message to client socket: %s", strerror(errno));
+	    }
+		
             speechd_connection_destroy(fd);
             /* This is internal Speech Dispatcher message, see serve() */
             spd_free(command);
@@ -148,38 +151,29 @@ parse(const char *buf, const int bytes, const int fd)
 
             /* Set the flag to command mode */
             MSG(5, "Switching back to command mode...");
-            SpeechdSocket[fd].awaiting_data = 0;
+            speechd_socket->awaiting_data = 0;
 
             /* Prepare element (text+settings commands) to be queued. */
 
 	    /* TODO: Remove? */
-            if ((bytes == 3) && (SpeechdSocket[fd].o_bytes > 2)) SpeechdSocket[fd].o_bytes -= 2;
+            if ((bytes == 3) && (speechd_socket->o_bytes > 2)) speechd_socket->o_bytes -= 2;
 
 	    /* Check if message contains any data */
-            if (SpeechdSocket[fd].o_bytes == 0) return strdup(OK_MSG_CANCELED);
+            if (speechd_socket->o_bytes == 0) return strdup(OK_MSG_CANCELED);
 
 	    /* Check buffer for proper UTF-8 encoding */
-	    if (!g_utf8_validate(SpeechdSocket[fd].o_buf->str, SpeechdSocket[fd].o_bytes, NULL))
+	    if (!g_utf8_validate(speechd_socket->o_buf->str, speechd_socket->o_bytes, NULL))
 	      {
-		MSG(3, "ERROR: Invalid character encoding on input (failed UTF-8 validation)");
-		MSG(3, "Rejecting this message.");
+		MSG(4, "ERROR: Invalid character encoding on input (failed UTF-8 validation)");
+		MSG(4, "Rejecting this message.");
 		return strdup(ERR_INVALID_ENCODING);
 	      }
 
             new = (TSpeechDMessage*) spd_malloc(sizeof(TSpeechDMessage));
-            new->bytes = SpeechdSocket[fd].o_bytes;
-	    new->buf = (char*) spd_malloc(new->bytes + 1);
-	    
-	    assert(SpeechdSocket[fd].o_buf != NULL);
-	    memcpy(new->buf, SpeechdSocket[fd].o_buf->str, new->bytes);
-	    new->buf[new->bytes] = 0;
-
-
-            reparted = SpeechdSocket[fd].inside_block; 
-
-	    /* TODO: Unify this with the above copying */
-	    new->buf = deescape_dot(new->buf);
-
+            new->bytes = speechd_socket->o_bytes;
+            assert(speechd_socket->o_buf != NULL);
+            new->buf = deescape_dot(speechd_socket->o_buf->str, new->bytes);
+            reparted = speechd_socket->inside_block; 
             MSG(5, "New buf is now: |%s|", new->buf);		
             if((msg_uid = queue_message(new, fd, 1, MSGTYPE_TEXT, reparted)) == 0){
                 if(SPEECHD_DEBUG) FATAL("Can't queue message\n");
@@ -212,9 +206,9 @@ parse(const char *buf, const int bytes, const int fd)
             }      
             /* Get the number of bytes read before, sum it with the number of bytes read
              * now and store again in the counter */        
-            SpeechdSocket[fd].o_bytes += real_bytes;       
+            speechd_socket->o_bytes += real_bytes;       
 
-            g_string_insert_len(SpeechdSocket[fd].o_buf, -1, buf, real_bytes);
+            g_string_insert_len(speechd_socket->o_buf, -1, buf, real_bytes);
         }
     }
 
@@ -228,7 +222,7 @@ parse(const char *buf, const int bytes, const int fd)
 
 #define CHECK_PARAM(param) \
     if (param == NULL){ \
-       MSG(3, "Missing parameter from client"); \
+       MSG(4, "Missing parameter from client"); \
        return strdup(ERR_MISSING_PARAMETER); \
     } 
 
@@ -237,7 +231,10 @@ parse(const char *buf, const int bytes, const int fd)
        char *helper; \
        helper = get_param(buf, pos, bytes, 0); \
        CHECK_PARAM(helper); \
-       if (!isanum(helper)) return strdup(ERR_NOT_A_NUMBER); \
+       if (!isanum(helper)){ \
+	   spd_free(helper); \
+	   return strdup(ERR_NOT_A_NUMBER); \
+       } \
        name = atoi(helper); \
        spd_free(helper); \
    }
@@ -256,7 +253,7 @@ parse(const char *buf, const int bytes, const int fd)
 
 /* Parses @history commands and calls the appropriate history_ functions. */
 char*
-parse_history(const char *buf, const int bytes, const int fd)
+parse_history(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     char *cmd_main;
     GET_PARAM_STR(cmd_main, 1, CONV_DOWN);
@@ -274,18 +271,19 @@ parse_history(const char *buf, const int bytes, const int fd)
         else if (TEST_CMD(hist_get_sub, "client_messages")){
             int start, num;
             char *who;
+	    int who_id;
 
             /* TODO: This needs to be (sim || am)-plified */
             who = get_param(buf,3,bytes, 1);
             CHECK_PARAM(who);
             if (!strcmp(who, "self")) return strdup(ERR_NOT_IMPLEMENTED);
-            if (!strcmp(who, "all")) return strdup(ERR_NOT_IMPLEMENTED);                          
-            if (!isanum(who)) return strdup(ERR_NOT_A_NUMBER);
-            
+            if (!strcmp(who, "all")) return strdup(ERR_NOT_IMPLEMENTED);                         
+            if (!isanum(who)) return strdup(ERR_NOT_A_NUMBER);           
+	    who_id = atoi(who);
+	    spd_free(who);	    
             GET_PARAM_INT(start, 4);
             GET_PARAM_INT(num, 5);
-
-            return (char*) history_get_message_list(atoi(who), start, num);
+            return (char*) history_get_message_list(who_id, start, num);
         }  
         else if (TEST_CMD(hist_get_sub, "last")){
             return (char*) history_get_last(fd);
@@ -383,7 +381,7 @@ parse_history(const char *buf, const int bytes, const int fd)
     }
 
 char*
-parse_set(const char *buf, const int bytes, const int fd)
+parse_set(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     int who;                    /* 0 - self, 1 - uid specified, 2 - all */
     int uid;                    /* uid of the client (only if who == 1) */
@@ -573,7 +571,7 @@ parse_set(const char *buf, const int bytes, const int fd)
     else SSIP_ON_OFF_PARAM(ssml_mode,
 	    OK_SSML_MODE_SET, ERR_COULDNT_SET_SSML_MODE,
 	    ALLOWED_INSIDE_BLOCK())
-      else SSIP_ON_OFF_PARAM(debug, g_strdup_printf("262-%s\r\n"OK_DEBUGGING, SpeechdOptions.debug_destination),
+    else SSIP_ON_OFF_PARAM(debug, g_strdup_printf("262-%s\r\n"OK_DEBUGGING, SpeechdOptions.debug_destination),
             ERR_COULDNT_SET_DEBUGGING, ;)
     else if (TEST_CMD(set_sub, "notification")){
 	char *scope;
@@ -587,7 +585,7 @@ parse_set(const char *buf, const int bytes, const int fd)
 
         if(TEST_CMD(par_s,"on")) par = 1;
         else if(TEST_CMD(par_s,"off")) par = 0;        
-        else{
+        else{	    
             spd_free(par_s);
             return strdup(ERR_PARAMETER_INVALID);
         }
@@ -599,6 +597,7 @@ parse_set(const char *buf, const int bytes, const int fd)
         return strdup(OK_NOTIFICATION_SET);
     }
     else{
+	spd_free(set_sub);
         return strdup(ERR_PARAMETER_INVALID);
     }
 
@@ -607,7 +606,7 @@ parse_set(const char *buf, const int bytes, const int fd)
 #undef SSIP_SET_COMMAND
 
 char*
-parse_stop(const char *buf, const int bytes, const int fd)
+parse_stop(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     int uid = 0;
     char *who_s;
@@ -645,7 +644,7 @@ parse_stop(const char *buf, const int bytes, const int fd)
 }
 
 char*
-parse_cancel(const char *buf, const int bytes, const int fd)
+parse_cancel(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     int uid = 0;
     char *who_s;
@@ -677,7 +676,7 @@ parse_cancel(const char *buf, const int bytes, const int fd)
 }
 
 char*
-parse_pause(const char *buf, const int bytes, const int fd)
+parse_pause(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     int uid = 0;
     char *who_s;
@@ -719,7 +718,7 @@ parse_pause(const char *buf, const int bytes, const int fd)
 }
 
 char*
-parse_resume(const char *buf, const int bytes, const int fd)
+parse_resume(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     int uid = 0;
     char *who_s;
@@ -750,7 +749,7 @@ parse_resume(const char *buf, const int bytes, const int fd)
 }
 
 char*
-parse_general_event(const char *buf, const int bytes, const int fd, EMessageType type)
+parse_general_event(const char *buf, const int bytes, const int fd, const TSpeechDSock* speechd_socket, EMessageType type)
 {
     char *param;
     TSpeechDMessage *msg;
@@ -768,8 +767,8 @@ parse_general_event(const char *buf, const int bytes, const int fd, EMessageType
     /* Check buffer for proper UTF-8 encoding */
     if (!g_utf8_validate(buf, bytes, NULL))
       {
-	MSG(3, "ERROR: Invalid character encoding on event input (failed UTF-8 validation)");
-	MSG(3, "Rejecting this event (char/key/sound_icon).");
+	MSG(4, "ERROR: Invalid character encoding on event input (failed UTF-8 validation)");
+	MSG(4, "Rejecting this event (char/key/sound_icon).");
 	return strdup(ERR_INVALID_ENCODING);
       }
 
@@ -778,7 +777,7 @@ parse_general_event(const char *buf, const int bytes, const int fd, EMessageType
     msg->bytes = strlen(param);
     msg->buf = strdup(param);
 
-    if(queue_message(msg, fd, 1, type, SpeechdSocket[fd].inside_block) == 0){
+    if(queue_message(msg, fd, 1, type, speechd_socket->inside_block) == 0){
         if (SPEECHD_DEBUG) FATAL("Couldn't queue message\n");
         MSG(2, "Error: Couldn't queue message!\n");            
     }   
@@ -789,25 +788,25 @@ parse_general_event(const char *buf, const int bytes, const int fd, EMessageType
 }
 
 char*
-parse_snd_icon(const char *buf, const int bytes, const int fd)
+parse_snd_icon(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
-    return parse_general_event(buf, bytes, fd, MSGTYPE_SOUND_ICON);
+    return parse_general_event(buf, bytes, fd, speechd_socket, MSGTYPE_SOUND_ICON);
 }				 
 
 char*
-parse_char(const char *buf, const int bytes, const int fd)
+parse_char(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
-    return parse_general_event(buf, bytes, fd, MSGTYPE_CHAR);
+    return parse_general_event(buf, bytes, fd, speechd_socket, MSGTYPE_CHAR);
 }
 
 char*
-parse_key(const char* buf, const int bytes, const int fd)
+parse_key(const char* buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
-    return parse_general_event(buf, bytes, fd, MSGTYPE_KEY);
+    return parse_general_event(buf, bytes, fd, speechd_socket, MSGTYPE_KEY);
 }
 
 char*
-parse_list(const char* buf, const int bytes, const int fd)
+parse_list(const char* buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     char *list_type;
     char *voice_list;
@@ -828,23 +827,24 @@ parse_list(const char* buf, const int bytes, const int fd)
                 OK_VOICE_LIST_SENT);
         return voice_list;
     }else if(TEST_CMD(list_type, "output_modules")){
-	GString *result;
+        GString *result = g_string_new("");
 	char *helper;
-	GList *gl;
-	int i;
-	int len;
-	result = g_string_new("");
-	len = g_list_length(output_modules_list);
-	MSG(1, "G LIST LENGHT IS %d", len);
-	for (i=0; i<=len-1; i++){
-	  gl = g_list_nth(output_modules_list, i);
-	  assert(gl!=NULL);
-	  if (gl->data == NULL) continue;
-	  g_string_append_printf(result, C_OK_MODULES"-%s\r\n", (char*) gl->data);
-	}
+	GList *cur_ptr = output_modules_list;
+
+	while(cur_ptr){
+	    if (cur_ptr->data != NULL){
+	        g_string_append_printf(result, C_OK_MODULES"-%s\r\n", (char*) cur_ptr->data);
+	    }else{
+                MSG(2, "null entry in output modules list");
+	    }
+
+            cur_ptr = cur_ptr->next;
+        }
+
 	g_string_append(result, OK_MODULES_LIST_SENT);
 	helper = result->str;
 	g_string_free(result, 0);
+
         return helper;
     }else if(TEST_CMD(list_type, "synthesis_voices")){
       char *module_name;
@@ -880,7 +880,7 @@ parse_list(const char* buf, const int bytes, const int fd)
 }
 
 char*
-parse_get(const char *buf, const int bytes, const int fd)
+parse_get(const char *buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     char *get_type;
     GString *result;
@@ -946,7 +946,7 @@ parse_get(const char *buf, const int bytes, const int fd)
 }
 
 char*
-parse_help(const char* buf, const int bytes, const int fd)
+parse_help(const char* buf, const int bytes, const int fd, const TSpeechDSock *speechd_socket)
 {
     char *help;
 
@@ -968,105 +968,99 @@ parse_help(const char* buf, const int bytes, const int fd)
 }
 
 char*
-parse_block(const char *buf, const int bytes, const int fd)
+parse_block(const char *buf, const int bytes, const int fd, TSpeechDSock* speechd_socket)
 {
     char *cmd_main;
     GET_PARAM_STR(cmd_main, 1, CONV_DOWN);
 
     if (TEST_CMD(cmd_main, "begin")){
-        assert(SpeechdSocket[fd].inside_block >= 0);
-        if (SpeechdSocket[fd].inside_block == 0){
-            SpeechdSocket[fd].inside_block = ++SpeechdStatus.max_gid;
+        assert(speechd_socket->inside_block >= 0);
+        if (speechd_socket->inside_block == 0){
+            speechd_socket->inside_block = ++SpeechdStatus.max_gid;
             return strdup(OK_INSIDE_BLOCK);
         }else{
             return strdup(ERR_ALREADY_INSIDE_BLOCK);
         }        
     }
     else if (TEST_CMD(cmd_main, "end")){
-        assert(SpeechdSocket[fd].inside_block >= 0);
-        if (SpeechdSocket[fd].inside_block > 0){
-            SpeechdSocket[fd].inside_block = 0;
+        assert(speechd_socket->inside_block >= 0);
+        if (speechd_socket->inside_block > 0){
+            speechd_socket->inside_block = 0;
             return strdup(OK_OUTSIDE_BLOCK);
         }else{
             return strdup(ERR_ALREADY_OUTSIDE_BLOCK);
-        }        
+        }
+    }
+    else{
+	spd_free(cmd_main);
+	return strdup(ERR_PARAMETER_INVALID);
     }
-    else return strdup(ERR_PARAMETER_INVALID);
 }
-
-/* TODO: I have no idea how this works. It seems it doesn't
-even work. */
-char*
-deescape_dot(char *otext)
-{
-    char *seq;
-    GString *ntext;
-    char *ootext;
-    char *ret = NULL;
-    int len;
-
-    if (otext == NULL) return NULL;
-
-    MSG2(6, "escaping", "Incomming text: |%s|", otext);
-
-    ootext = otext;
-
-    ntext = g_string_new("");
-
-    if (strlen(otext) == 2){
-        if (!strcmp(otext, "..")){
-            otext[1] = 0;
-	    g_string_free(ntext,1);
-            return otext;
-        }
-    }
-
-    if (strlen(otext) >= 2){
-        if ((otext[0] == '.') && (otext[1] == '.')){
-            g_string_append(ntext, ".");
-            otext = otext+2;
-        }
-    }
-
-    MSG2(6, "escaping", "Altering text (I): |%s|", ntext->str);
-
-    while ( (seq = strstr(otext, "\r\n..\r\n")) ){
-        *seq = 0;
-        g_string_append(ntext, otext);
-        g_string_append(ntext, "\r\n.\r\n");
-
-        MSG2(6, "escaping", "Altering text (II) / 1: |%s|", otext);    
-        otext = seq + 6;
-        MSG2(6, "escaping", "Altering text (II) / 2: |%s|", otext);    
-    }
-
-    MSG2(6, "escaping", "Altering text (II): |%s|", ntext->str);    
-
-    len = strlen(otext);
-    if (len >= 4){
-        if ((otext[len-4] == '\r') && (otext[len-3] == '\n')
-            && (otext[len-2] == '.') && (otext[len-1] == '.')){
-            otext[len-1] = 0;
-            MSG2(6, "escaping", "Altering text (II-b) otext: |%s|", otext);    
-        }
-    }
-
-    if (otext == ootext){
-        ret = otext;
-	g_string_free(ntext, 1);
-    }else{
-        g_string_append(ntext, otext);
-        spd_free(ootext);
-        ret = ntext->str;
-	g_string_free(ntext, 0);
-    }
-
-    MSG2(6, "escaping", "Altered text: |%s|", ret);
-
-    return ret;
-}
-
    
+/*
+     * deescape_dot: Replace .. with . at the start of lines or at the
+     * start of the string.
+ * @orig_text: text to be unescaped.
+  * @orig_len: length of the text.
+ * Returns: a freshly allocated string, containing the unescaped data.
+ *
+ * In SSIP, the message terminator is \r\n.\r\n, just as it is in SMTP
+ * and similar protocols.  Thus, period needs to be escaped when it
+ * is the only character on a line.  deescape_dot reverts that
+ * transformation, after the message is received.
+ * This function deserves further examination.
+ */
+
+char*
+deescape_dot(const char *orig_text, size_t orig_len)
+{
+    /* Constants.  DOTLINE is CRLF followed by a period.
+     * DOTLINELEN is the length of DOTLINE.
+     * ESCAPED_DOTLINELEN is the length of the sequence \r\n..,
+     * which is used in the original (unescaped) text.
+     */
+    static const char *DOTLINE = "\r\n.";
+    static const size_t DOTLINELEN = 3;
+    static const size_t ESCAPED_DOTLINELEN = 4;	/* \r\n.. */
+
+    char *out_text = NULL;
+    char *out_ptr;
+    const char *orig_end = orig_text + orig_len;
+
+    if (orig_text == NULL)
+        return NULL;
+
+    out_text = spd_malloc(orig_len + 1);
+    /* We may have allocated more than we need.  In any case, out_text
+     * can be no longer than orig_text.
+     * Note: spd_malloc aborts the program on failure to allocate. */
+
+    out_ptr = out_text;
+    if (orig_len >= 2) {
+        /* De-escape .. at start of text. */
+        if ((orig_text[0] == '.') && (orig_text[1] == '.')) {
+            *(out_ptr++) = '.';
+            orig_text = orig_text+2;
+        }
+    }
+
+    while (orig_text < orig_end) {
+        if ((orig_text[0] == '\r') && (orig_text[1] == '\n')
+                && (orig_text[2] == '.') && (orig_text[3] == '.')) {
+            /* We just found \r\n.., the sequence we want to unescape. */
+            memcpy(out_ptr, DOTLINE, DOTLINELEN);
+            out_ptr += DOTLINELEN;
+            orig_text += ESCAPED_DOTLINELEN;
+        } else {
+            /* Just copy the character from source to destination... */
+            *(out_ptr++) = *(orig_text++);
+        }
+    }
+
+    *out_ptr = '\0';	/* NUL-terminate. */
+    return out_text;
+}
+
 /* isanum() tests if the given string is a number,
  * returns 1 if yes, 0 otherwise. */
 int
