@@ -139,10 +139,8 @@ speak(void* data)
        MSG(5, "Locking element_free_mutex in speak()");
        pthread_mutex_lock(&element_free_mutex);
         /* Handle postponed priority progress message */
-       
        check_locked(&element_free_mutex);
-
-        if ((g_list_length(last_p5_block) != 0) && (g_list_length(MessageQueue->p5) == 0)){      
+       if ((g_list_length(last_p5_block) != 0) && (g_list_length(MessageQueue->p5) == 0)){      
 	    /* Transfer messages from last_p5_block to priority 3 (message) queue*/
 	    while (g_list_length(last_p5_block) != 0){
 		GList* item;
@@ -151,6 +149,7 @@ speak(void* data)
 		check_locked(&element_free_mutex);
 		MessageQueue->p3 = g_list_insert_sorted(MessageQueue->p3, message, sortbyuid);
 		last_p5_block = g_list_remove_link(last_p5_block, item);
+		g_list_free1(item);
 	    }
 	    assert(message!=NULL);
             highest_priority = 3;
@@ -209,11 +208,28 @@ speak(void* data)
 
         /* Set the id of the client who is speaking. */
         speaking_uid = message->settings.uid;
-	if (current_message != NULL)
-	    if (!current_message->settings.paused_while_speaking)
+	if (current_message != NULL){
+	    if (!current_message->settings.paused_while_speaking){
+		/* Check if the client who emited this message is disconnected
+		   by now and this was his last message. If so, delete it's settings
+		   from fdset */
+		if (get_client_settings_by_uid(current_message->settings.uid)->active == 0){
+		  if (!client_has_messages(current_message->settings.uid)
+		      && (current_message->settings.uid != message->settings.uid)){
+			/* client_has_messages does not account for message,
+			   which was just retrieved from the queue.
+			   We also have to compare the uids of message
+			   and current_message to be sure that there are
+			   no outstanding messages. */
+			MSG(4, "Removing client settings for uid %d", current_message->settings.uid);
+			remove_client_settings_by_uid(current_message->settings.uid);
+		    }
+		}
 		mem_free_message(current_message);
+	    }	
+	}
         current_message = message;
-
+	
         /* Check if the last priority 5 message wasn't said yet */
         if (last_p5_block != NULL){
 	    GList *elem;
@@ -443,12 +459,12 @@ speaking_pause(int fd, int uid)
     TFDSetElement *settings;
     int ret;
 
-    MSG(3, "Pause");
+    MSG(4, "Pause");
 
     /* Find settings for this particular client */
     settings = get_client_settings_by_uid(uid);
     if (settings == NULL){
-	MSG(3, "ERROR: Can't get settings of active client in speaking_pause()!");
+	MSG(4, "ERROR: Can't get settings of active client in speaking_pause()!");
 	return 1;
     }
     settings->paused = 1;
@@ -921,6 +937,7 @@ TSpeechDMessage*
 get_message_from_queues()
 {
     GList *gl = NULL;
+    TSpeechDMessage * message;
 
     /* We will descend through priorities to say more important
      * messages first. */
@@ -967,7 +984,34 @@ get_message_from_queues()
         }
     } 
     assert(gl != NULL);
-    return (TSpeechDMessage *) gl->data;
+   
+    message = gl->data;
+    g_list_free(gl);
+    return message;
+}
+
+gint
+message_has_uid(gconstpointer msg, gconstpointer uid)
+{
+    if (((TSpeechDMessage*) msg)->settings.uid == *(int*)uid)
+	return 0;
+    else
+	return 1;
+}
+
+/* Return 1 if any message from this client is found
+   in any of the queues, otherwise return 0 */    
+int
+client_has_messages(int uid){
+    
+    if (g_list_find_custom(MessageQueue->p5, (gconstpointer) &uid, message_has_uid) ||
+	g_list_find_custom(MessageQueue->p4, (gconstpointer) &uid, message_has_uid) ||
+	g_list_find_custom(MessageQueue->p3, (gconstpointer) &uid, message_has_uid) ||
+	g_list_find_custom(MessageQueue->p2, (gconstpointer) &uid, message_has_uid) ||
+	g_list_find_custom(MessageQueue->p1, (gconstpointer) &uid, message_has_uid))
+	return 1;
+    else
+	return 0;       
 }
 
 GList*
