@@ -21,7 +21,9 @@
  * $Id: speechd.c,v 1.81 2008-07-10 15:36:49 hanke Exp $
  */
 
+#include <glib.h>
 #include <gmodule.h>
+#include <glib/gstdio.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -40,8 +42,6 @@
 #include "options.h"
 #include "server.h"
 
-/* Glib g_makedir */
-int  g_mkdir      (const gchar *filename, int mode);
 /* Manipulating pid files */
 int create_pid_file();
 void destroy_pid_file();
@@ -201,7 +201,10 @@ MSG2(int level, char *kind, char *format, ...)
 }
 
 /* The main logging function for Speech Dispatcher,
-   level is between 1 and 5. 1 means the most important.*/
+   level is between -1 and 5. 1 means the most important,
+   5 less important. Loglevels after 4 can contain private
+   data. -1 logs also to stderr. See Speech Dispatcher
+   documentation */
 /* TODO: Define this in terms of MSG somehow. I don't
    know how to pass '...' arguments to another C function */
 void
@@ -214,8 +217,8 @@ MSG(int level, char *format, ...)
         int i;
 	pthread_mutex_lock(&logging_mutex);		
         {
-            {
-                /* Print timestamp */
+	    /* Print timestamp */
+	    {
                 time_t t;
                 char *tstr;
 		struct timeval tv;
@@ -225,7 +228,9 @@ MSG(int level, char *format, ...)
                 /* Remove the trailing \n */
 		assert(tstr);
 		assert(strlen(tstr)>1);
+		assert((level>=-1) && (level<=5));
 		tstr[strlen(tstr)-1] = 0;
+		/* Write timestamps */
 		if (level <= SpeechdOptions.log_level)
 		  fprintf(logfile, "[%s : %d] speechd: ",
 			  tstr, (int) tv.tv_usec);
@@ -238,8 +243,9 @@ MSG(int level, char *format, ...)
             }
 	    
             for(i=1;i<level;i++){
-	      fprintf(logfile, " ");
+                fprintf(logfile, " ");
             }
+	    /* Log to ordinary logfile */
 	    if (level <= SpeechdOptions.log_level){
 	      va_start(args, format);	    
 	      vfprintf(logfile, format, args);
@@ -247,6 +253,7 @@ MSG(int level, char *format, ...)
 	      fprintf(logfile, "\n");
 	      fflush(logfile);
 	    }
+	    /* Log into debug logfile */
 	    if (SpeechdOptions.debug){
 	      va_start(args, format);	   
 	      vfprintf(debug_logfile, format, args);
@@ -254,6 +261,14 @@ MSG(int level, char *format, ...)
 	      fprintf(debug_logfile, "\n");
 	      fflush(debug_logfile);
 	    }
+	    /* Log also into stderr for loglevel -1*/
+	    if (level == -1){
+	      va_start(args, format);	    
+	      vfprintf(stderr, format, args);
+	      va_end(args);
+	      fprintf(stderr, "\n"); 
+	      fflush(stderr);
+	    }
         }
 	pthread_mutex_unlock(&logging_mutex);		
     }				
@@ -261,6 +276,59 @@ MSG(int level, char *format, ...)
 
 /* --- CLIENTS / CONNECTIONS MANAGING --- */
 
+/* Initialize sockets status table */
+int
+speechd_sockets_status_init(void)
+{
+    speechd_sockets_status = g_hash_table_new_full(g_int_hash, g_int_equal,
+						   (GDestroyNotify) spd_free,
+						   (GDestroyNotify) speechd_socket_free);
+    if (speechd_sockets_status)
+	return 0;
+    else
+	return 1;
+}
+
+/* Register a new socket for SSIP connection */
+int
+speechd_socket_register(int fd)
+{
+    int *fd_key;
+    TSpeechDSock* speechd_socket;
+    speechd_socket = spd_malloc(sizeof(TSpeechDSock));
+    speechd_socket->o_buf = NULL;
+    speechd_socket->o_bytes = 0;
+    speechd_socket->awaiting_data = 0;
+    speechd_socket->inside_block = 0;
+    fd_key = spd_malloc(sizeof(int));
+    *fd_key = fd;
+    g_hash_table_insert(speechd_sockets_status, fd_key, speechd_socket);
+    return 0;
+}
+
+/* Free a TSpeechDSock structure including it's data */
+void
+speechd_socket_free(TSpeechDSock* speechd_socket)
+{
+    if (speechd_socket->o_buf)
+	g_string_free(speechd_socket->o_buf, 1);
+    spd_free(speechd_socket);
+}
+
+/* Unregister a socket for SSIP communication */
+int
+speechd_socket_unregister(int fd)
+{
+    return (!g_hash_table_remove(speechd_sockets_status, &fd));
+}
+
+/* Get a pointer to the TSpeechDSock structure for a given file descriptor */
+TSpeechDSock*
+speechd_socket_get_by_fd(int fd)
+{
+    return g_hash_table_lookup(speechd_sockets_status, &fd);
+}
+
 /* activity is on server_socket (request for a new connection) */
 int
 speechd_connection_new(int server_socket)
@@ -269,7 +337,7 @@ speechd_connection_new(int server_socket)
     struct sockaddr_in client_address;
     unsigned int client_len = sizeof(client_address);
     int client_socket;
-    int *p_client_socket, *p_client_uid;
+    int *p_client_socket, *p_client_uid, *p_client_uid2;
 
     client_socket = accept(server_socket, (struct sockaddr *)&client_address,
                            &client_len);
@@ -284,18 +352,7 @@ speechd_connection_new(int server_socket)
     if (client_socket > SpeechdStatus.max_fd) SpeechdStatus.max_fd = client_socket;
     MSG(4,"Adding client on fd %d", client_socket);
 
-    /* Check if there is space for server status data; allocate it */
-    if(client_socket >= SpeechdStatus.num_fds-1){
-	SpeechdSocket = (TSpeechdSock*) realloc(SpeechdSocket,
-						SpeechdStatus.num_fds*2*
-						sizeof(TSpeechdSock)); 
-        SpeechdStatus.num_fds *= 2;
-    }
-
-    SpeechdSocket[client_socket].o_buf = g_string_new("");
-    SpeechdSocket[client_socket].o_bytes = 0;
-    SpeechdSocket[client_socket].awaiting_data = 0;
-    SpeechdSocket[client_socket].inside_block = 0;
+    speechd_socket_register(client_socket);
 
     /* Create a record in fd_settings */
     new_fd_set = (TFDSetElement *) default_fd_set();
@@ -309,12 +366,13 @@ speechd_connection_new(int server_socket)
     new_fd_set->uid = ++SpeechdStatus.max_uid;
     p_client_socket = (int*) spd_malloc(sizeof(int));
     p_client_uid = (int*) spd_malloc(sizeof(int));
+    p_client_uid2 = (int*) spd_malloc(sizeof(int));
     *p_client_socket = client_socket;
     *p_client_uid = SpeechdStatus.max_uid;
+    *p_client_uid2 = SpeechdStatus.max_uid;
 
     g_hash_table_insert(fd_settings, p_client_uid, new_fd_set);
-
-    g_hash_table_insert(fd_uid, p_client_socket, p_client_uid);
+    g_hash_table_insert(fd_uid, p_client_socket, p_client_uid2);
 		
 
     MSG(4,"Data structures for client on fd %d created", client_socket);
@@ -334,28 +392,29 @@ speechd_connection_destroy(int fd)
 	if(fdset_element != NULL){
 		fdset_element->fd = -1;
 		fdset_element->active = 0;
+		/* The fdset_element will be freed and removed from the
+		   hash table as soon as the client no longer has any
+		   message in the queues, check out the speak() function */
 	}else if(SPEECHD_DEBUG){
 	 	DIE("Can't find settings for this client\n");
 	}
 
 	MSG(4,"Removing client from the fd->uid table.");
-	g_hash_table_remove(fd_uid, &fd);
-		
-        SpeechdSocket[fd].awaiting_data = 0;
-        SpeechdSocket[fd].inside_block = 0;
-	if (SpeechdSocket[fd].o_buf != NULL)
-	    g_string_free(SpeechdSocket[fd].o_buf, 1);
+
+	g_hash_table_remove(fd_uid, &fd);       
+
+	speechd_socket_unregister(fd);
 
 	MSG(4,"Closing clients file descriptor %d", fd);
 
 	if(close(fd) != 0)
 		if(SPEECHD_DEBUG)
                     DIE("Can't close file descriptor associated to this client");
-	   	
+
 	FD_CLR(fd, &readfds);
 	if (fd == SpeechdStatus.max_fd) SpeechdStatus.max_fd--;
 
-        MSG(3,"Connection closed");
+        MSG(4,"Connection closed");
 
 	return 0;
 }
@@ -373,10 +432,11 @@ speechd_client_terminate(gpointer key, gpointer value, gpointer user)
 	}	
 
 	if(set->fd>0){
-		MSG(4,"Closing connection on fd %d\n", set->fd);
+	        MSG(4,"Closing connection on fd %d\n", set->fd);
 		speechd_connection_destroy(set->fd);
 	}
 	mem_free_fdset(set);
+	spd_free(set);
 	return TRUE;
 }
 
@@ -481,7 +541,7 @@ speechd_options_init(void)
     SpeechdOptions.spawn = FALSE;
     SpeechdOptions.log_level_set = 0;
     SpeechdOptions.communication_method = NULL;
-    SpeechdOptions.socket_name = NULL;
+    SpeechdOptions.socket_path = NULL;
     SpeechdOptions.port_set = 0;
     SpeechdOptions.localhost_access_only_set = 0;
     SpeechdOptions.pid_file = NULL;
@@ -498,11 +558,8 @@ speechd_options_init(void)
 void
 speechd_init()
 {
-    int START_NUM_FD = 16;
     int ret;
-    int i;
     
-
     SpeechdStatus.max_uid = 0;
     SpeechdStatus.max_gid = 0;
 
@@ -523,10 +580,14 @@ speechd_init()
     output_modules_list = NULL;
 
     /* Initialize hash tables */
-    fd_settings = g_hash_table_new(g_int_hash,g_int_equal);
+    fd_settings = g_hash_table_new_full(g_int_hash, g_int_equal,
+					(GDestroyNotify) spd_free,
+					NULL);
     assert(fd_settings != NULL);
 
-    fd_uid = g_hash_table_new(g_str_hash, g_str_equal);
+    fd_uid = g_hash_table_new_full(g_int_hash, g_int_equal,
+				   (GDestroyNotify) spd_free,
+				   (GDestroyNotify) spd_free);
     assert(fd_uid != NULL);
 
     language_default_modules = g_hash_table_new(g_str_hash, g_str_equal);
@@ -535,13 +596,7 @@ speechd_init()
     output_modules = g_hash_table_new(g_str_hash, g_str_equal);
     assert(output_modules != NULL);
 
-    SpeechdSocket = (TSpeechdSock*) spd_malloc(START_NUM_FD * sizeof(TSpeechdSock));
-    SpeechdStatus.num_fds = START_NUM_FD;
-    for(i=0; i<=START_NUM_FD-1; i++){
-        SpeechdSocket[i].awaiting_data = 0;              
-        SpeechdSocket[i].inside_block = 0;              
-        SpeechdSocket[i].o_buf = 0;              
-    }
+    speechd_sockets_status_init();
 
     pause_requested = 0;
     resume_requested = 0;
@@ -573,7 +628,7 @@ speechd_init()
       SpeechdOptions.log_dir = g_strdup_printf("%s/log/", SpeechdOptions.home_speechd_dir);
       mkdir(SpeechdOptions.log_dir, S_IRWXU);
       if (!SpeechdOptions.debug_destination){
-	SpeechdOptions.debug_destination = g_strdup_printf("%slog/debug",
+	SpeechdOptions.debug_destination = g_strdup_printf("%s/log/debug",
 							   SpeechdOptions.home_speechd_dir);
 	mkdir(SpeechdOptions.debug_destination, S_IRWXU);
       }     
@@ -665,7 +720,7 @@ speechd_quit(int sig)
 	MSG(2, "close() failed: %s", strerror(errno));
     FD_CLR(server_socket, &readfds);  
     
-    MSG(3, "Removing pid file");
+    MSG(4, "Removing pid file");
     destroy_pid_file();
     
     fflush(NULL);
@@ -699,13 +754,13 @@ create_pid_file()
         /* If there is a lock, exit, otherwise remove the old file */
         ret = fcntl(pid_fd, F_GETLK, &lock);
         if (ret == -1){
-            MSG(1, "Can't check lock status of an existing pid file.\n");
+            MSG(-1, "Can't check lock status of an existing pid file.\n");
             return -1;
         }
 
         fclose(pid_file);
         if (lock.l_type != F_UNLCK){
-            MSG(2, "Speech Dispatcher already running.\n");
+            MSG(-1, "Speech Dispatcher already running.\n");
             return -1;
         }
 
@@ -715,7 +770,7 @@ create_pid_file()
     /* Create a new pid file and lock it */
     pid_file = fopen(SpeechdOptions.pid_file, "w");
     if (pid_file == NULL){
-        MSG(1, "Can't create pid file in %s, wrong permissions?\n",
+        MSG(-1, "Can't create pid file in %s, wrong permissions?\n",
 	    SpeechdOptions.pid_file);
         return -1;
     }
@@ -730,7 +785,7 @@ create_pid_file()
 
     ret = fcntl(pid_fd, F_SETLK, &lock);
     if (ret == -1){
-        MSG(1, "Can't set lock on pid file.\n");
+        MSG(-1, "Can't set lock on pid file.\n");
         return -1;
     }
 
@@ -759,7 +814,7 @@ logging_init(void)
 	      file_name);
       logfile = stdout;
     } else {
-      MSG(2,"Speech Dispatcher Logging to file %s", file_name);
+      MSG(3,"Speech Dispatcher Logging to file %s", file_name);
     }
   }
 
@@ -796,7 +851,7 @@ make_local_socket (const char *filename)
   }
 
   if (listen(sock, 50) == -1){
-    MSG(2,"ERRNO:%s", strerror(errno));
+    MSG(2,"listen failed: ERRNO:%s", strerror(errno));
     FATAL("listen() failed for local socket");
   }
 
@@ -832,10 +887,10 @@ make_inet_socket(const int port)
   
   server_address.sin_port = htons(port);
 
-  MSG(3,"Openning inet socket connection");
+  MSG(4,"Openning inet socket connection");
   if (bind(server_socket, (struct sockaddr *)&server_address,
 	   sizeof(server_address)) == -1){
-    MSG(1, "bind() failed: %s", strerror(errno));
+    MSG(-1, "bind() failed: %s", strerror(errno));
     FATAL("Couldn't open inet socket, try a few minutes later.");
   }
 
@@ -855,6 +910,10 @@ main(int argc, char *argv[])
     fd_set testfds;
     int fd;
     int ret;
+    /* Autospawn helper variables */
+    char *spawn_communication_method = NULL;
+    int spawn_port = 0;
+    char *spawn_socket_path = NULL;
 
     /* Initialize threading and thread safety in Glib */
     g_thread_init(NULL);
@@ -863,7 +922,7 @@ main(int argc, char *argv[])
     umask(007);
 
     /* Initialize logging */
-    logfile = stderr;
+    logfile = stdout;
     SpeechdOptions.log_level = 1;
     custom_logfile = NULL;
     custom_log_kind = NULL;
@@ -872,6 +931,26 @@ main(int argc, char *argv[])
 
     options_parse(argc, argv);
 
+    if (SpeechdOptions.spawn){
+      /* In case of --spawn, copy the host port and socket_path
+	 parameters into temporary spawn_ variables for later comparison
+	 with the config file and unset them*/
+      if (SpeechdOptions.communication_method_set){
+	spawn_communication_method = strdup(SpeechdOptions.communication_method);
+	spd_free(SpeechdOptions.communication_method);
+	SpeechdOptions.communication_method_set = 0;
+      }
+      if (SpeechdOptions.port_set){
+	spawn_port = SpeechdOptions.port;
+	SpeechdOptions.port_set = 0;
+      }
+      if (SpeechdOptions.socket_path_set){
+	spawn_socket_path = strdup(SpeechdOptions.socket_path);
+	spd_free(SpeechdOptions.socket_path);
+	SpeechdOptions.socket_path_set = 0;
+      }
+    }
+
     MSG(1, "Speech Dispatcher "VERSION" starting");
 
     /* By default, search for configuration options and put everything
@@ -926,13 +1005,13 @@ main(int argc, char *argv[])
 
       err = g_file_get_contents(SpeechdOptions.conf_file, &config_contents, NULL, NULL);
       if (err == FALSE){
-	MSG(1,"Error openning %s", SpeechdOptions.conf_file);
+	MSG(-1,"Error openning %s", SpeechdOptions.conf_file);
 	FATAL("Can't open conf file");
       }
       regexp = g_regex_new ("^[ ]*DisableAutoSpawn", G_REGEX_MULTILINE,0,NULL);
       result = g_regex_match(regexp, config_contents, 0, NULL);      
       if (result){
-	MSG(4,"Autospawn requested but disabled in configuration");	
+	MSG(-1,"Autospawn requested but disabled in configuration");	
 	exit(1);
       }
       g_free(config_contents);
@@ -957,42 +1036,90 @@ main(int argc, char *argv[])
 
     speechd_init();
 
+    /* Handle socket_path 'default' */
+    // TODO: This is a hack, we should do that at appropriate places...
+    if (!strcmp(SpeechdOptions.socket_path, "default")){
+      /* This code cannot be moved above next to conf_dir and pidpath resolution because
+	 we need to also consider the DotConf configuration, which is read in speechd_init() */
+      GString *socket_filename;
+      socket_filename = g_string_new("");
+      if (SpeechdOptions.home_speechd_dir){
+	g_string_printf(socket_filename, "%s/speechd.sock", SpeechdOptions.home_speechd_dir);
+      }else{
+	FATAL("Socket name file not set and user has no home directory");
+      }
+      spd_free(SpeechdOptions.socket_path);
+      SpeechdOptions.socket_path = strdup(socket_filename->str);
+      g_string_free(socket_filename, 1);
+    }
+
+
+    /* Check if the communication method corresponds to the spawn request */
+    /* TODO: This should preferably be done much sooner, but the current
+       configuration mechanism doesn't allow it */
+    if (SpeechdOptions.spawn){
+      if (spawn_communication_method){
+	if (strcmp(spawn_communication_method, SpeechdOptions.communication_method)){
+	  MSG(-1, "Autospawn failed: Mismatch in communication methods. Client " \
+	      "requests %s, most probably due to its configuration or the value of " \
+	      "the SPEECHD_ADDRESS environment variable, but the server is configured " \
+	      "to provide the %s method.",
+	      spawn_communication_method, SpeechdOptions.communication_method);
+	  exit(1);
+	}else{
+	  if (!strcmp(SpeechdOptions.communication_method, "inet_socket")){
+	    /* Check port */
+	    if (spawn_port != 0)
+	      if (spawn_port != SpeechdOptions.port){
+		MSG(-1, "Autospawn failed: Mismatch in port numbers. Server " \
+		    "is configured to use the inet_socket method on the port %d " \
+		    "while the client requests port %d, most probably due to its " \
+		    "configuration or the value of the SPEECHD_ADDRESS environment "
+		    "variable.",
+		    SpeechdOptions.port, spawn_port);
+		exit(1);
+	      }
+	  }else if (!strcmp(SpeechdOptions.communication_method, "unix_socket")){
+	    /* Check socket name */
+	    if (spawn_socket_path)
+	      if (strcmp(spawn_socket_path, SpeechdOptions.socket_path)){
+		MSG(-1, "Autospawn failed: Mismatch in socket names. The server " \
+		    "is configured to provide a socket interface in %s, but the " \
+		    "client requests a different path: %s. This is most probably " \
+		    "due to the client application configuration or the value of " \
+		    "the SPEECHD_ADDRESS environment variable.",
+		    SpeechdOptions.socket_path, spawn_socket_path);
+		exit(1);
+	      }
+	  } else assert (0);
+	}
+      }
+      spd_free(spawn_communication_method);
+      spd_free(spawn_socket_path);
+    }
+
     if(!strcmp(SpeechdOptions.communication_method, "inet_socket")){
-      MSG(2, "Speech Dispatcher will use inet port %d", SpeechdOptions.port);
+      MSG(4, "Speech Dispatcher will use inet port %d", SpeechdOptions.port);
       /* Connect and start listening on inet socket */
       server_socket = make_inet_socket(SpeechdOptions.port);
     }else if (!strcmp(SpeechdOptions.communication_method, "unix_socket")){
       /* Determine appropariate socket file name */
-      GString *socket_filename;
-      if (!strcmp(SpeechdOptions.socket_name, "default")){
-	/* This code cannot be moved above next to conf_dir and pidpath resolution because
-	 we need to also consider the DotConf configuration, which is read in speechd_init() */
-	socket_filename = g_string_new("");
-	if (SpeechdOptions.home_speechd_dir){
-	  g_string_printf(socket_filename, "%s/speechd.sock", SpeechdOptions.home_speechd_dir);
-	}else{
-	  FATAL("Socket name file not set and user has no home directory");
-	}
-      }else{
-	socket_filename = g_string_new(SpeechdOptions.socket_name);
-      }
-      MSG(2, "Speech Dispatcher will use local unix socket: %s", socket_filename->str);
-
+      MSG(4, "Speech Dispatcher will use local unix socket: %s", SpeechdOptions.socket_path);
       /* Delete an old socket file if it exists */
-      if (g_file_test(socket_filename->str, G_FILE_TEST_EXISTS))
-	  if (g_unlink(socket_filename->str) == -1)
-	    FATAL("Local socket file exists but impossible to delete. Wrong permissions?");
-      
+      if (g_file_test(SpeechdOptions.socket_path, G_FILE_TEST_EXISTS))
+	  if (g_unlink(SpeechdOptions.socket_path) == -1)
+	    FATAL("Local socket file exists but impossible to delete. Wrong permissions?");      
       /* Connect and start listening on local unix socket */
-      server_socket = make_local_socket(socket_filename->str);
-      g_string_free(socket_filename, 1);
+      server_socket = make_local_socket(SpeechdOptions.socket_path);
     }else{
       FATAL("Unknown communication method");
     }
 
     /* Fork, set uid, chdir, etc. */
     if (spd_mode == SPD_MODE_DAEMON){
-        daemon(0,0);
+        if (daemon(0,0)){
+	    FATAL("Can't fork child process");
+        }
         /* Re-create the pid file under this process */
         unlink(SpeechdOptions.pid_file);
         if (create_pid_file() == -1) return -1;
@@ -1013,7 +1140,10 @@ main(int argc, char *argv[])
 
         if (select(FD_SETSIZE, &testfds, (fd_set *)0, (fd_set *)0, NULL) >= 1){
             /* Once we know we've got activity,
-             * we find which descriptor it's on by checking each in turn using FD_ISSET. */
+             * we find which descriptor it's on by checking each in turn using FD_ISSET.
+	     TODO: This is a hack. We should not be browsing all possible socket numbers
+	     but only those which are connected.
+	     */
             for (fd = 0; fd <= SpeechdStatus.max_fd && fd < FD_SETSIZE; fd++) {
                 if (FD_ISSET(fd,&testfds)){
                     MSG(4,"Activity on fd %d ...",fd);
