@@ -71,7 +71,6 @@ union semun {
 #include "def.h"
 #include "fdset.h"
 #include "module.h"
-#include "parse.h"
 #include "compare.h"
 
 /* Size of the buffer for socket communication */
@@ -111,8 +110,8 @@ typedef struct{
 struct{
     char *communication_method;
     int communication_method_set;
-    char *socket_name;
-    int socket_name_set;
+    char *socket_path;
+    int socket_path_set;
     int port, port_set;
     int localhost_access_only, localhost_access_only_set;
     int log_level, log_level_set;
@@ -132,7 +131,6 @@ struct{
     int max_uid;		/* The largest assigned uid + 1 */
     int max_gid;		/* The largest assigned gid + 1 */
     int max_fd;
-    int num_fds;		/* Number of available allocated sockets */
 }SpeechdStatus;
 
 /* speak() thread defined in speaking.c */
@@ -181,20 +179,27 @@ fd_set readfds;
 /* Inter thread comm pipe */
 int speaking_pipe[2];
 
-/* Arrays needed for receiving data over socket */
+
+/* Managing sockets communication */
+GHashTable *speechd_sockets_status;
 typedef struct{
     int awaiting_data;
     int inside_block;
     size_t o_bytes;
     GString *o_buf;
-}TSpeechdSock;
+}TSpeechDSock;
+int speechd_sockets_status_init(void);
+int speechd_socket_register(int fd);
+void speechd_socket_free(TSpeechDSock* speechd_socket);
+int speechd_socket_unregister(int fd);
+TSpeechDSock* speechd_socket_get_by_fd(int fd);
 
-TSpeechdSock *SpeechdSocket;
+#include "parse.h"
 
 /* Debugging */
 void MSG(int level, char *format, ...);
 void MSG2(int level, char* kind, char *format, ...);
-#define FATAL(msg) { fatal_error(); MSG(0,"Fatal error [%s:%d]:"msg, __FILE__, __LINE__); exit(EXIT_FAILURE); }
+#define FATAL(msg) { fatal_error(); MSG(-1,"Fatal error [%s:%d]:"msg, __FILE__, __LINE__); exit(EXIT_FAILURE); }
 #define DIE(msg) { MSG(0,"Error [%s:%d]:"msg, __FILE__, __LINE__); exit(EXIT_FAILURE); }
 
 FILE *logfile;
